@@ -77,7 +77,7 @@ static int parse_extensions_egprs(const uint8_t *data, unsigned int data_len,
 			/* TS 44.060, table 10.4.14a.1, row 3 & 5 */
 			/* only filling bytes left */
 			LOGP(DRLCMACUL, LOGL_DEBUG, "UL DATA LI contains "
-				"only filling bytes with extention octet: LI=%d, E=%d, count=%d\n",
+				"only filling bytes with extension octet: LI=%d, E=%d, count=%d\n",
 				li->li, li->e, num_chunks);
 			break;
 		} else if (li->li > 0) {
@@ -92,7 +92,7 @@ static int parse_extensions_egprs(const uint8_t *data, unsigned int data_len,
 		}
 
 		LOGP(DRLCMACUL, LOGL_DEBUG, "UL DATA LI contains "
-			"extention octet: LI=%d, E=%d, count=%d\n",
+			"extension octet: LI=%d, E=%d, count=%d\n",
 			li->li, li->e, num_chunks);
 		num_chunks += 1;
 
@@ -168,7 +168,7 @@ static int parse_extensions_gprs(const uint8_t *data, unsigned int data_len,
 		chunks[num_chunks].is_complete = li->li || is_last_block;
 
 		LOGP(DRLCMACUL, LOGL_DEBUG, "UL DATA LI contains "
-			"extention octet: LI=%d, M=%d, E=%d, count=%d\n",
+			"extension octet: LI=%d, M=%d, E=%d, count=%d\n",
 			li->li, li->m, li->e, num_chunks);
 		num_chunks += 1;
 
@@ -215,7 +215,7 @@ int Decoding::rlc_data_from_ul_data(
 				"but no more chunks possible\n");
 			return -ENOSPC;
 		}
-	} else if (cs.isEgprs()) {
+	} else if (mcs_is_edge(cs)) {
 		/* if E is not set (LI follows), EGPRS */
 		num_chunks = parse_extensions_egprs(data, data_len, &offs,
 			is_last_block,
@@ -240,7 +240,7 @@ int Decoding::rlc_data_from_ul_data(
 		}
 
 		memcpy(&tlli_enc, data + offs, sizeof(tlli_enc));
-		if (cs.isGprs())
+		if (mcs_is_gprs(cs))
 			/* The TLLI is encoded in big endian for GPRS (see
 			 * TS 44.060, figure 10.2.2.1, note) */
 			*tlli = be32toh(tlli_enc);
@@ -355,22 +355,22 @@ int Decoding::rlc_parse_ul_data_header(struct gprs_rlc_data_info *rlc,
 {
 	unsigned int cur_bit = 0;
 	switch(cs.headerTypeData()) {
-	case GprsCodingScheme::HEADER_GPRS_DATA :
+	case HEADER_GPRS_DATA :
 		cur_bit = rlc_parse_ul_data_header_gprs(rlc, data, cs);
 		break;
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_3 :
+	case HEADER_EGPRS_DATA_TYPE_3 :
 		cur_bit = rlc_parse_ul_data_header_egprs_type_3(rlc, data, cs);
 		break;
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_2 :
+	case HEADER_EGPRS_DATA_TYPE_2 :
 		cur_bit = rlc_parse_ul_data_header_egprs_type_2(rlc, data, cs);
 		break;
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_1 :
+	case HEADER_EGPRS_DATA_TYPE_1 :
 		cur_bit = rlc_parse_ul_data_header_egprs_type_1(rlc, data, cs);
 		break;
 	default:
 		LOGP(DRLCMACDL, LOGL_ERROR,
 			"Decoding of uplink %s data blocks not yet supported.\n",
-			cs.name());
+			mcs_name(cs));
 		return -ENOTSUP;
 	};
 
