@@ -23,20 +23,112 @@
 #include <gprs_rlcmac.h>
 #include <bts.h>
 #include <tbf.h>
+#include <tbf_ul.h>
 #include <gprs_debug.h>
 #include <egprs_rlc_compression.h>
 
 extern "C" {
 #include <osmocom/gprs/protocol/gsm_04_60.h>
-}
-
 #include <osmocom/gsm/protocol/gsm_04_08.h>
+}
 
 #include <stdbool.h>
 #include <errno.h>
 #include <string.h>
 
+#define CHECK(rc) { if (rc < 0) return rc; }
+#define SET_X(bv, x) { if (bitvec_set_bit(bv, x) < 0) return -EOWNERDEAD; }
+#define SET_0(bv) SET_X(bv, ZERO)
+#define SET_1(bv) SET_X(bv, ONE)
+#define SET_L(bv) SET_X(bv, L)
+#define SET_H(bv) SET_X(bv, H)
+
+/* 3GPP TS 44.018 § 10.5.2.16:
+   { 0 | 1 < ALPHA : bit (4) > }
+   < GAMMA : bit (5) >
+*/
+static int write_alpha_gamma(bitvec *dest, uint8_t alpha, uint8_t gamma)
+{
+	int rc;
+
+	if (alpha) {
+		SET_1(dest);
+		rc = bitvec_set_u64(dest, alpha, 4, false);
+		CHECK(rc);
+	} else
+		SET_0(dest);
+
+	rc = bitvec_set_u64(dest, gamma, 5, false);
+	CHECK(rc);
+
+	return 0;
+}
+
+/* TBF_STARTING_TIME -- same as 3GPP TS 44.018 §10.5.2.38 Starting Time without tag: */
+static int write_tbf_start_time(bitvec *dest, uint32_t fn)
+{
+	int rc;
+
+	/* Set values according to 3GPP TS 44.018 Table 10.5.2.38.1 */
+
+	/* T1' */
+	rc = bitvec_set_u64(dest, (fn / (26 * 51)) % 32, 5, false);
+	CHECK(rc);
+
+	/* T3  */
+	rc = bitvec_set_u64(dest, fn % 51, 6, false);
+	CHECK(rc);
+
+	/* T2  */
+	rc = bitvec_set_u64(dest, fn % 26, 5, false);
+	CHECK(rc);
+
+	return rc;
+}
+
+/* 3GPP TS 44.018 §10.5.2.16:
+   < TFI_ASSIGNMENT : bit (5) >
+   < POLLING : bit >
+   0 -- The value '1' was allocated in an earlier version of the protocol and shall not be used.
+   < USF: bit (3) >
+   < USF_GRANULARITY : bit >
+   { 0 | 1 < P0 : bit (4) > < PR_MODE : bit (1) > }
+*/
+static int write_tfi_usf(bitvec *dest, const gprs_rlcmac_ul_tbf *tbf, uint8_t usf)
+{
+	int rc = bitvec_set_u64(dest, tbf->tfi(), 5, false); /* TFI_ASSIGNMENT */
+	CHECK(rc);
+
+	SET_0(dest); /* POLLING -- no action is required from MS */
+
+	SET_0(dest);
+
+	rc = bitvec_set_u64(dest, usf, 3, false); /* USF */
+	CHECK(rc);
+
+	SET_0(dest); /* USF_GRANULARITY -- the mobile station shall transmit one RLC/MAC block */
+
+	SET_0(dest); /* No P0 nor PR_MODE */
+
+	return 0;
+}
+
 /* { 0 | 1 < TIMING_ADVANCE_INDEX : bit (4) > } */
+static int write_ta_index(bitvec *dest, int8_t tai)
+{
+	int rc;
+
+	if (tai < 0) { /* No TIMING_ADVANCE_INDEX: */
+		SET_0(dest);
+	} else { /* TIMING_ADVANCE_INDEX: */
+		SET_1(dest);
+		rc = bitvec_set_u64(dest, tai, 4, false);
+		CHECK(rc);
+	}
+
+	return 0;
+}
+
 static inline bool write_tai(bitvec *dest, unsigned& wp, int8_t tai)
 {
 	if (tai < 0) { /* No TIMING_ADVANCE_INDEX: */
@@ -88,168 +180,176 @@ static inline void write_ta_ie(bitvec *dest, unsigned& wp,
 		bitvec_write_field(dest, &wp, ts, 3);
 }
 
-static int write_ia_rest_downlink(
-	gprs_rlcmac_dl_tbf *tbf,
-	bitvec * dest, unsigned& wp,
-	uint8_t polling, bool ta_valid, uint32_t fn,
-	uint8_t alpha, uint8_t gamma, int8_t ta_idx)
+static int write_ia_rest_downlink(const gprs_rlcmac_dl_tbf *tbf, bitvec * dest, bool polling, bool ta_valid,
+				  uint32_t fn, uint8_t alpha, uint8_t gamma, int8_t ta_idx)
 {
-	if (!tbf) {
-		LOGP(DRLCMACDL, LOGL_ERROR,
-			"Cannot encode DL IMMEDIATE ASSIGNMENT without TBF\n");
-		return -EINVAL;
-	}
-	// GSM 04.08 10.5.2.16 IA Rest Octets
-	bitvec_write_field(dest, &wp, 3, 2);   // "HH"
-	bitvec_write_field(dest, &wp, 1, 2);   // "01" Packet Downlink Assignment
-	bitvec_write_field(dest, &wp,tbf->tlli(),32); // TLLI
-	bitvec_write_field(dest, &wp,0x1,1);   // switch TFI   : on
-	bitvec_write_field(dest, &wp,tbf->tfi(),5);   // TFI
-	bitvec_write_field(dest, &wp,0x0,1);   // RLC acknowledged mode
-	if (alpha) {
-		bitvec_write_field(dest, &wp,0x1,1);   // ALPHA = present
-		bitvec_write_field(dest, &wp,alpha,4);   // ALPHA
-	} else {
-		bitvec_write_field(dest, &wp,0x0,1);   // ALPHA = not present
-	}
-	bitvec_write_field(dest, &wp,gamma,5);   // GAMMA power control parameter
-	bitvec_write_field(dest, &wp,polling,1);   // Polling Bit
-	bitvec_write_field(dest, &wp, ta_valid, 1); // N. B: NOT related to TAI!
-	write_tai(dest, wp, ta_idx);
+	int rc = 0;
+
+	SET_H(dest); SET_H(dest);
+	SET_0(dest); SET_1(dest); /* 00 Packet Downlink Assignment */
+
+	rc = bitvec_set_u64(dest, tbf->tlli(), 32, false); /* TLLI */
+	CHECK(rc);
+
+	SET_1(dest);
+	rc = bitvec_set_u64(dest, tbf->tfi(), 5, false);   /* TFI_ASSIGNMENT */
+	CHECK(rc);
+
+	/* RLC acknowledged mode */
+	SET_0(dest); /* RLC_MODE */
+
+	rc = write_alpha_gamma(dest, alpha, gamma);
+	CHECK(rc);
+
+	rc = bitvec_set_bit(dest, (bit_value) polling); /* POLLING */
+	CHECK(rc);
+
+	/* N. B: NOT related to TAI! */
+	rc = bitvec_set_bit(dest, (bit_value) ta_valid); /* TA_VALID */
+	CHECK(rc);
+
+	rc = write_ta_index(dest, ta_idx);
+	CHECK(rc);
+
 	if (polling) {
-		bitvec_write_field(dest, &wp,0x1,1);   // TBF Starting TIME present
-		bitvec_write_field(dest, &wp,(fn / (26 * 51)) % 32,5); // T1'
-		bitvec_write_field(dest, &wp,fn % 51,6);               // T3
-		bitvec_write_field(dest, &wp,fn % 26,5);               // T2
-	} else {
-		bitvec_write_field(dest, &wp,0x0,1);   // TBF Starting TIME present
-	}
-	bitvec_write_field(dest, &wp,0x0,1);   // P0 not present
-	//		bitvec_write_field(dest, &wp,0x1,1);   // P0 not present
-	//		bitvec_write_field(dest, &wp,,0xb,4);
+		SET_1(dest);
+		rc = write_tbf_start_time(dest, fn);
+		CHECK(rc);
+	} else
+		SET_0(dest);
+
+	SET_0(dest); /* No P0 nor PR_MODE */
+
 	if (tbf->is_egprs_enabled()) {
-		/* see GMS 44.018, 10.5.2.16 */
-		bitvec_write_field(dest, &wp, 1, 1);  // "H"
-		write_ws(dest, &wp, tbf->window_size()); // EGPRS Window Size
-		bitvec_write_field(dest, &wp, 0x0, 2);    // LINK_QUALITY_MEASUREMENT_MODE
-		bitvec_write_field(dest, &wp, 0, 1);      // BEP_PERIOD2 not present
-	}
+		SET_H(dest);
+		rc = bitvec_set_u64(dest, enc_ws(tbf->window_size()), 5, false); /* EGPRS Window Size */
+		CHECK(rc);
 
-	return 0;
+		/* The mobile station shall not report measurements: (see 3GPP TS 44.060 Table 11.2.7.1) */
+		SET_0(dest); SET_0(dest); /* LINK_QUALITY_MEASUREMENT_MODE */
+		SET_1(dest);              /* No BEP_PERIOD2 */
+	} else
+		SET_L(dest);              /* No Additions for Rel-6 */
+
+	return rc;
 }
 
-static int write_ia_rest_uplink(
-	gprs_rlcmac_ul_tbf *tbf,
-	bitvec * dest, unsigned& wp,
-	uint8_t usf, uint32_t fn,
-	uint8_t alpha, uint8_t gamma, int8_t ta_idx)
+/* 3GPP TS 44.018 Table 10.5.2.16.1 < Packet Uplink Assignment > -- Single Block Allocation */
+static int write_ia_rest_uplink_sba(bitvec *dest, uint32_t fn, uint8_t alpha, uint8_t gamma)
 {
-	OSMO_ASSERT(!tbf || !tbf->is_egprs_enabled());
+	int rc = 0;
 
-	// GMS 04.08 10.5.2.37b 10.5.2.16
-	bitvec_write_field(dest, &wp, 3, 2);    // "HH"
-	bitvec_write_field(dest, &wp, 0, 2);    // "0" Packet Uplink Assignment
-	if (tbf == NULL) {
-		bitvec_write_field(dest, &wp, 0, 1);    // Block Allocation : Single Block Allocation
-		if (alpha) {
-			bitvec_write_field(dest, &wp,0x1,1);   // ALPHA = present
-			bitvec_write_field(dest, &wp,alpha,4);   // ALPHA = present
-		} else
-			bitvec_write_field(dest, &wp,0x0,1);   // ALPHA = not present
-		bitvec_write_field(dest, &wp,gamma,5);   // GAMMA power control parameter
-		write_tai(dest, wp, ta_idx);
-		bitvec_write_field(dest, &wp, 1, 1);    // TBF_STARTING_TIME_FLAG
-		bitvec_write_field(dest, &wp,(fn / (26 * 51)) % 32,5); // T1'
-		bitvec_write_field(dest, &wp,fn % 51,6);               // T3
-		bitvec_write_field(dest, &wp,fn % 26,5);               // T2
-	} else {
-		bitvec_write_field(dest, &wp, 1, 1);    // Block Allocation : Not Single Block Allocation
-		bitvec_write_field(dest, &wp, tbf->tfi(), 5);  // TFI_ASSIGNMENT Temporary Flow Identity
-		bitvec_write_field(dest, &wp, 0, 1);    // POLLING
-		bitvec_write_field(dest, &wp, 0, 1);    // ALLOCATION_TYPE: dynamic
-		bitvec_write_field(dest, &wp, usf, 3);    // USF
-		bitvec_write_field(dest, &wp, 0, 1);    // USF_GRANULARITY
-		bitvec_write_field(dest, &wp, 0, 1);   // "0" power control: Not Present
-		bitvec_write_field(dest, &wp, tbf->current_cs().to_num()-1, 2);    // CHANNEL_CODING_COMMAND
-		bitvec_write_field(dest, &wp, 1, 1);    // TLLI_BLOCK_CHANNEL_CODING
-		if (alpha) {
-			bitvec_write_field(dest, &wp,0x1,1);   // ALPHA = present
-			bitvec_write_field(dest, &wp,alpha,4);   // ALPHA
-		} else
-			bitvec_write_field(dest, &wp,0x0,1);   // ALPHA = not present
-		bitvec_write_field(dest, &wp,gamma,5);   // GAMMA power control parameter
-		/* note: there is no choise for TAI and no starting time */
-		bitvec_write_field(dest, &wp, 0, 1);   // switch TIMING_ADVANCE_INDEX = off
-		bitvec_write_field(dest, &wp, 0, 1);    // TBF_STARTING_TIME_FLAG
-	}
-	return 0;
+	SET_0(dest); /* Single Block Allocation */
+	rc = write_alpha_gamma(dest, alpha, gamma);
+	CHECK(rc);
+
+	/* A 'Timing Advance index' shall not be allocated at a Single Block allocation.
+	   A 'TBF Starting Time' shall be allocated at a Single Block allocation. */
+	SET_0(dest);
+	SET_1(dest);
+
+	rc = write_tbf_start_time(dest, fn);
+	CHECK(rc);
+
+	 /* No P0 nor PR_MODE */
+	SET_L(dest);
+
+	/* No Additions for R99 */
+	SET_L(dest);
+
+	 /* No Additions for Rel-6 */
+	SET_L(dest);
+
+	return rc;
 }
 
-static int write_ia_rest_egprs_uplink(
-	gprs_rlcmac_ul_tbf *tbf,
-	bitvec * dest, unsigned& wp,
-	uint8_t usf, uint32_t fn,
-	uint8_t alpha, uint8_t gamma, int8_t ta_idx,
-	enum ph_burst_type burst_type, uint16_t ra)
+static int write_ia_rest_uplink_mba(const gprs_rlcmac_ul_tbf *tbf, bitvec *dest, uint8_t usf,
+				    uint8_t alpha, uint8_t gamma, int8_t ta_idx)
 {
-	uint8_t extended_ra = 0;
+	int rc = 0;
 
-	extended_ra = (ra & 0x1F);
+	SET_1(dest); /* Multi Block Allocation */
 
-	bitvec_write_field(dest, &wp, 1, 2);    /* LH */
-	bitvec_write_field(dest, &wp, 0, 2);    /* 0 EGPRS Uplink Assignment */
-	bitvec_write_field(dest, &wp, extended_ra, 5);    /* Extended RA */
-	bitvec_write_field(dest, &wp, 0, 1);    /* Access technology Request */
+	rc = write_tfi_usf(dest, tbf, usf);
+	CHECK(rc);
 
-	if (tbf == NULL) {
+	/* 3GPP TS 44.060 Table 11.2.28.2 Channel Coding Indicator */
+	rc = bitvec_set_u64(dest, mcs_chan_code(tbf->current_cs()), 2, false); /* CHANNEL_CODING_COMMAND */
+	CHECK(rc);
 
-		bitvec_write_field(dest, &wp, 0, 1); /* multiblock allocation */
+	/* TLLI_BLOCK_CHANNEL_CODING */
+	SET_1(dest); /* use coding scheme as specified by the corresponding CHANNEL CODING COMMAND */
 
-		if (alpha) {
-			bitvec_write_field(dest, &wp, 0x1, 1); /* ALPHA =yes */
-			bitvec_write_field(dest, &wp, alpha, 4); /* ALPHA */
-		} else {
-			bitvec_write_field(dest, &wp, 0x0, 1); /* ALPHA = no */
-		}
+	rc = write_alpha_gamma(dest, alpha, gamma);
+	CHECK(rc);
 
-		bitvec_write_field(dest, &wp, gamma, 5); /* GAMMA power contrl */
-		bitvec_write_field(dest, &wp, (fn / (26 * 51)) % 32, 5);/* T1' */
-		bitvec_write_field(dest, &wp, fn % 51, 6);              /* T3 */
-		bitvec_write_field(dest, &wp, fn % 26, 5);              /* T2 */
-		bitvec_write_field(dest, &wp, 0, 2); /* Radio block allocation */
+	rc = write_ta_index(dest, ta_idx);
+	CHECK(rc);
 
-		bitvec_write_field(dest, &wp, 0, 1);
+	/* No TBF_STARTING_TIME */
+	SET_0(dest);
 
-	} else {
-		bitvec_write_field(dest, &wp, 1, 1);     /* single block alloc */
-		bitvec_write_field(dest, &wp, tbf->tfi(), 5);/* TFI assignment */
-		bitvec_write_field(dest, &wp, 0, 1);     /* polling bit */
-		bitvec_write_field(dest, &wp, 0, 1);     /* constant */
-		bitvec_write_field(dest, &wp, usf, 3);   /* USF bit */
-		bitvec_write_field(dest, &wp, 0, 1);     /* USF granularity */
-		bitvec_write_field(dest, &wp, 0, 1);     /* P0 */
-		/* MCS */
-		bitvec_write_field(dest, &wp, tbf->current_cs().to_num()-1, 4);
-		/* tlli channel block */
-		bitvec_write_field(dest, &wp, tbf->tlli(), 1);
-		bitvec_write_field(dest, &wp, 0, 1);   /* BEP period present */
-		bitvec_write_field(dest, &wp, 0, 1);   /* resegmentation */
-		write_ws(dest, &wp, tbf->window_size()); /* EGPRS window size */
+	return rc;
+}
 
-		if (alpha) {
-			bitvec_write_field(dest, &wp, 0x1, 1);   /* ALPHA =yes */
-			bitvec_write_field(dest, &wp, alpha, 4); /* ALPHA */
-		} else {
-			bitvec_write_field(dest, &wp, 0x0, 1);   /* ALPHA = no */
-		}
+static int write_ia_rest_egprs_uplink_mba(bitvec * dest, uint32_t fn, uint8_t alpha, uint8_t gamma)
+{
+	int rc = 0;
 
-		bitvec_write_field(dest, &wp, gamma, 5); /* GAMMA power contrl */
-		bitvec_write_field(dest, &wp, 0, 1); /* TIMING_ADVANCE_INDEX */
-		bitvec_write_field(dest, &wp, 0, 1); /* TBF_STARTING_TIME_FLAG */
-		bitvec_write_field(dest, &wp, 0, 1); /* NULL */
-	}
+	SET_0(dest); /* Multi Block Allocation */
 
-	return 0;
+	rc = write_alpha_gamma(dest, alpha, gamma);
+	CHECK(rc);
+
+	rc = write_tbf_start_time(dest, fn);
+	CHECK(rc);
+
+	SET_0(dest); /* NUMBER OF RADIO BLOCKS ALLOCATED: */
+	SET_0(dest); /* 1 radio block reserved for uplink transmission */
+	SET_0(dest); /* No P0 */
+
+	return rc;
+}
+
+static int write_ia_rest_egprs_uplink_sba(const gprs_rlcmac_ul_tbf *tbf, bitvec * dest, uint8_t usf,
+					  uint8_t alpha, uint8_t gamma, int8_t ta_idx)
+{
+	int rc = 0;
+
+	SET_1(dest); /* Single Block Allocation */
+
+	rc = write_tfi_usf(dest, tbf, usf);
+	CHECK(rc);
+
+	/* 3GPP TS 44.060 §12.10d EGPRS Modulation and coding Scheme description: */
+	rc = bitvec_set_u64(dest, mcs_chan_code(tbf->current_cs()), 4, false); /* EGPRS CHANNEL_CODING_COMMAND */
+	CHECK(rc);
+
+	/* TLLI_BLOCK_CHANNEL_CODING */
+	rc = bitvec_set_bit(dest, (bit_value)tbf->tlli());
+
+	/* No BEP_PERIOD2 */
+	SET_0(dest);
+
+	/* Retransmitted RLC data blocks shall not be re-segmented: (see 3GPP TS 44.060 §12.10e) */
+	SET_0(dest); /* RESEGMENT */
+
+	rc = bitvec_set_u64(dest, enc_ws(tbf->window_size()), 5, false); /* EGPRS Window Size */
+	CHECK(rc);
+
+	rc = write_alpha_gamma(dest, alpha, gamma);
+	CHECK(rc);
+
+	rc = write_ta_index(dest, ta_idx);
+	CHECK(rc);
+
+	/* No TBF_STARTING_TIME */
+	SET_0(dest);
+
+	/* No Additions for Rel-7 */
+	SET_0(dest);
+
+	return rc;
 }
 
 /*
@@ -343,10 +443,10 @@ static inline void log_alert_exit(const char * error)
  */
 int Encoding::write_immediate_assignment(
 	struct gprs_rlcmac_tbf *tbf,
-	bitvec * dest, uint8_t downlink, uint16_t ra,
+	bitvec * dest, bool downlink, uint16_t ra,
 	uint32_t ref_fn, uint8_t ta, uint16_t arfcn, uint8_t ts, uint8_t tsc,
-	uint8_t usf, uint8_t polling, uint32_t fn, uint8_t alpha,
-	uint8_t gamma, int8_t ta_idx, enum ph_burst_type burst_type, uint8_t sb)
+	uint8_t usf, bool polling, uint32_t fn, uint8_t alpha,
+	uint8_t gamma, int8_t ta_idx, enum ph_burst_type burst_type)
 {
 	unsigned wp = 0;
 	int plen;
@@ -397,23 +497,43 @@ int Encoding::write_immediate_assignment(
 
 	plen = wp / 8;
 
-	if (downlink)
-		rc = write_ia_rest_downlink(as_dl_tbf(tbf), dest, wp,
-					    polling, gsm48_ta_is_valid(ta), fn,
-			alpha, gamma, ta_idx);
-	else if (((burst_type == GSM_L1_BURST_TYPE_ACCESS_1) ||
-			(burst_type == GSM_L1_BURST_TYPE_ACCESS_2)))
-		rc = write_ia_rest_egprs_uplink(as_ul_tbf(tbf), dest, wp,
-			usf, fn,
-			alpha, gamma, ta_idx, burst_type, ra);
-	else
-		rc = write_ia_rest_uplink(as_ul_tbf(tbf), dest, wp,
-			usf, fn,
-			alpha, gamma, ta_idx);
+	/* 3GPP TS 44.018 §10.5.2.16 IA Rest Octets */
+	dest->cur_bit = wp;
+	if (downlink) {
+		if (!as_dl_tbf(tbf)) {
+			LOGP(DRLCMACDL, LOGL_ERROR, "Cannot encode DL IMMEDIATE ASSIGNMENT without TBF\n");
+			return -EINVAL;
+		}
+
+		rc = write_ia_rest_downlink(as_dl_tbf(tbf), dest, polling, gsm48_ta_is_valid(ta), fn, alpha, gamma,
+					    ta_idx);
+	} else if (((burst_type == GSM_L1_BURST_TYPE_ACCESS_1) || (burst_type == GSM_L1_BURST_TYPE_ACCESS_2))) {
+		SET_L(dest); SET_H(dest); // "LH"
+		SET_0(dest); SET_0(dest); // "00" < EGPRS Packet Uplink Assignment >
+		rc = bitvec_set_u64(dest, ra & 0x1F, 5, false); // < Extended RA >
+		CHECK(rc);
+
+		SET_0(dest); // No < Access Technologies Request struct >
+
+		if (as_ul_tbf(tbf) != NULL)
+			rc = write_ia_rest_egprs_uplink_sba(as_ul_tbf(tbf), dest, usf, alpha, gamma, ta_idx);
+		else
+			rc = write_ia_rest_egprs_uplink_mba(dest, fn, alpha, gamma);
+	} else {
+		OSMO_ASSERT(!tbf || !tbf->is_egprs_enabled());
+
+		SET_H(dest); SET_H(dest); // "HH"
+		SET_0(dest); SET_0(dest); // "00" < Packet Uplink Assignment >
+
+		if (as_ul_tbf(tbf) != NULL)
+			rc = write_ia_rest_uplink_mba(as_ul_tbf(tbf), dest, usf, alpha, gamma, ta_idx);
+		else
+			rc = write_ia_rest_uplink_sba(dest, fn, alpha, gamma);
+	}
 
 	if (rc < 0) {
 		LOGP(DRLCMAC, LOGL_ERROR,
-			"Failed to create IMMEDIATE ASSIGMENT (%s) for %s\n",
+			"Failed to create IMMEDIATE ASSIGNMENT (%s) for %s\n",
 			downlink ? "downlink" : "uplink",
 			tbf ? tbf->name() : "single block allocation");
 		return rc;
@@ -424,7 +544,6 @@ int Encoding::write_immediate_assignment(
 
 /* generate uplink assignment */
 void Encoding::write_packet_uplink_assignment(
-	struct gprs_rlcmac_bts *bts,
 	bitvec * dest, uint8_t old_tfi,
 	uint8_t old_downlink, uint32_t tlli, uint8_t use_tlli,
 	struct gprs_rlcmac_ul_tbf *tbf, uint8_t poll, uint8_t rrbp, uint8_t alpha,
@@ -456,7 +575,7 @@ void Encoding::write_packet_uplink_assignment(
 
 	if (!use_egprs) {
 		bitvec_write_field(dest, &wp,0x0,1); // Message escape
-		bitvec_write_field(dest, &wp,tbf->current_cs().to_num()-1, 2); // CHANNEL_CODING_COMMAND
+		bitvec_write_field(dest, &wp, mcs_chan_code(tbf->current_cs()), 2); // CHANNEL_CODING_COMMAND
 		bitvec_write_field(dest, &wp,0x1,1); // TLLI_BLOCK_CHANNEL_CODING
 		write_ta_ie(dest, wp,tbf->ta(), ta_idx, ta_ts);
 	} else { /* EPGRS */
@@ -464,7 +583,7 @@ void Encoding::write_packet_uplink_assignment(
 		bitvec_write_field(dest, &wp,0x0,2); // EGPRS message contents
 		bitvec_write_field(dest, &wp,0x0,1); // No CONTENTION_RESOLUTION_TLLI
 		bitvec_write_field(dest, &wp,0x0,1); // No COMPACT reduced MA
-		bitvec_write_field(dest, &wp,tbf->current_cs().to_num()-1, 4); // EGPRS Modulation and Coding IE
+		bitvec_write_field(dest, &wp, mcs_chan_code(tbf->current_cs()), 4); // EGPRS Modulation and Coding IE
 		/* 0: no RESEGMENT, 1: Segmentation*/
 		bitvec_write_field(dest, &wp, 0x1, 1);
 		write_ws(dest, &wp, tbf->window_size()); // EGPRS Window Size
@@ -668,7 +787,7 @@ void Encoding::encode_rbb(const char *show_rbb, uint8_t *rbb)
 }
 
 static void write_packet_ack_nack_desc_gprs(
-	struct gprs_rlcmac_bts *bts, bitvec * dest, unsigned& wp,
+	bitvec * dest, unsigned& wp,
 	gprs_rlc_ul_window *window, bool is_final)
 {
 	char rbb[65];
@@ -690,12 +809,12 @@ static void write_packet_ack_nack_desc_gprs(
 }
 
 static void write_packet_uplink_ack_gprs(
-	struct gprs_rlcmac_bts *bts, bitvec * dest, unsigned& wp,
+	bitvec * dest, unsigned& wp,
 	struct gprs_rlcmac_ul_tbf *tbf, bool is_final)
 {
 
-	bitvec_write_field(dest, &wp, tbf->current_cs().to_num() - 1, 2); // CHANNEL_CODING_COMMAND
-	write_packet_ack_nack_desc_gprs(bts, dest, wp, tbf->window(), is_final);
+	bitvec_write_field(dest, &wp, mcs_chan_code(tbf->current_cs()), 2); // CHANNEL_CODING_COMMAND
+	write_packet_ack_nack_desc_gprs(dest, wp, tbf->window(), is_final);
 
 	bitvec_write_field(dest, &wp, 1, 1); // 1: have CONTENTION_RESOLUTION_TLLI
 	bitvec_write_field(dest, &wp, tbf->tlli(), 32); // CONTENTION_RESOLUTION_TLLI
@@ -710,46 +829,57 @@ static void write_packet_uplink_ack_gprs(
 	bitvec_write_field(dest, &wp, 0, 1); // 0: don't have REL 5
 };
 
+/* Encode the Ack/Nack for EGPRS. 44.060
+ * The PCU encodes to receive block bitmap to the following rules:
+ * - always encode the lenght field
+ * - use compressed receive block bitmap if it's smaller than uncompressed
+ *   receive block bitmap
+ * - use the remaining bits for an uncompressed receive block bitmap if needed
+ *
+ * Note: The spec also defines an Ack/Nack without length field, but the PCU's
+ *       doesn't support this in UL. It would require a lot more code complexity
+ *       and only saves 7 bit in lossy sitations.
+ */
 static void write_packet_ack_nack_desc_egprs(
-	struct gprs_rlcmac_bts *bts, bitvec * dest, unsigned& wp,
-	gprs_rlc_ul_window *window, bool is_final, unsigned& rest_bits)
+	bitvec * dest, unsigned& wp,
+	gprs_rlc_ul_window *window, bool is_final, unsigned rest_bits)
 {
 	unsigned int urbb_len = 0;
 	uint8_t crbb_len = 0;
 	uint8_t len;
 	bool bow = true;
 	bool eow = true;
-	int ssn = window->mod_sns(window->v_q() + 1);
+	uint16_t ssn = window->mod_sns(window->v_q() + 1);
 	unsigned int num_blocks = window->mod_sns(window->v_r() - window->v_q());
-	int esn_crbb = window->mod_sns(ssn - 1);
+	uint16_t esn_crbb = window->mod_sns(ssn - 1);
 	static  uint8_t rbb[RLC_EGPRS_MAX_WS] = {'\0'};
 	uint8_t iter = 0;
 	int is_compressed = 0;
 	bool try_compression = false;
-	uint8_t ucmp_bmplen;
+	uint16_t ucmp_bmplen;
 	uint8_t crbb_bitmap[23] = {'\0'};
 	bitvec ucmp_vec;
 	bitvec crbb_vec;
 	uint8_t uclen_crbb = 0;
-	bool len_coded = true;
 	uint8_t crbb_start_clr_code;
 	uint8_t i;
-#if 0
-	/* static size of 16 bits*/
-	 ..0. .... = ACKNACK:  (Union)
-        Desc
 
-            ...0 .... = FINAL_ACK_INDICATION: False
+	/* static size of 16 bits
+	 ..1. .... = ACKNACK:  (Union)
+	    0 0000 000 Length
+	Desc
 
-            .... 1... = BEGINNING_OF_WINDOW: 1
+	    ...0 .... = FINAL_ACK_INDICATION: False
 
-            .... .1.. = END_OF_WINDOW: 1
+	    .... 1... = BEGINNING_OF_WINDOW: 1
 
-            .... ..10  0101 0001  1... .... = STARTING_SEQUENCE_NUMBER: 1187
+	    .... .1.. = END_OF_WINDOW: 1
 
-            .0.. .... = CRBB Exist: 0
-#endif
-	rest_bits -= 16;
+	    .... ..10  0101 0001  1... .... = STARTING_SEQUENCE_NUMBER: 1187
+
+	    .0.. .... = CRBB Exist: 0
+	minimal size is 24 rest_bits */
+	rest_bits -= 24;
 
 	if (num_blocks > 0)
 		/* V(Q) is NACK and omitted -> SSN = V(Q) + 1 */
@@ -786,47 +916,31 @@ static void write_packet_ack_nack_desc_egprs(
 		crbb_len = crbb_vec.cur_bit;
 	}
 
-	if (is_compressed == 0) {
-		/* length field takes 8 bits*/
-		if (num_blocks > rest_bits - 8) {
-			eow = false;
-			urbb_len = rest_bits;
-			len_coded = false;
-		} else if (num_blocks == rest_bits) {
-			urbb_len = rest_bits;
-			len_coded = false;
-		} else
-			urbb_len = num_blocks;
 
-		len = urbb_len + 15;
+	if (is_compressed) {
+		/* 8 = 7 (CRBBlength) + 1 (CRBB starting color code) */
+		rest_bits -= 8;
 	} else {
-		if (num_blocks > uclen_crbb) {
-			eow = false;
-			urbb_len = num_blocks - uclen_crbb;
-		}
-		/* Union bit takes 1 bit */
-		/* Other fields in descr of compresed bitmap takes 23 bits
-		 * -8 = CRBB_STARTING_COLOR_CODE + CRBB_LENGTH */
-		if (urbb_len > (rest_bits - crbb_len - 8)) {
-			eow = false;
-			len_coded = false;
-			urbb_len = rest_bits - crbb_len - 8;
-		/* -16 =  ACKNACK Dissector length + CRBB_STARTING_COLOR_CODE + CRBB_LENGTH */
-		} else if (urbb_len > (rest_bits - crbb_len - 16)) {
-			eow = false;
-			len_coded = false;
-			urbb_len = rest_bits - crbb_len - 16;
-		}
+		uclen_crbb = 0;
+		crbb_len = 0;
+	}
+
+	if (num_blocks > uclen_crbb + rest_bits) {
+		eow = false;
+		urbb_len = rest_bits - crbb_len;
+	} else
+		urbb_len = num_blocks - uclen_crbb;
+
+	if (is_compressed)
 		len = urbb_len + crbb_len + 23;
-	}
+	else
+		len = urbb_len + 15;
 
-	/* EGPRS Ack/Nack Description IE */
-	if (len_coded == false) {
-		bitvec_write_field(dest, &wp, 0, 1); // 0: don't have length
-	} else {
-		bitvec_write_field(dest, &wp, 1, 1); // 1: have length
-		bitvec_write_field(dest, &wp, len, 8); // length
-	}
+
+	/* EGPRS Ack/Nack Description IE
+	 * do not support Ack/Nack without length */
+	bitvec_write_field(dest, &wp, 1, 1); // 1: have length
+	bitvec_write_field(dest, &wp, len, 8); // length
 
 	bitvec_write_field(dest, &wp, is_final, 1); // FINAL_ACK_INDICATION
 	bitvec_write_field(dest, &wp, bow, 1); // BEGINNING_OF_WINDOW
@@ -846,7 +960,7 @@ static void write_packet_ack_nack_desc_egprs(
 				crbb_len = crbb_len - 8;
 				iter++;
 			} else {
-				bitvec_write_field(dest, &wp, crbb_bitmap[iter], crbb_len);
+				bitvec_write_field(dest, &wp, crbb_bitmap[iter] >> (8 - crbb_len), crbb_len);
 				crbb_len = 0;
 			}
 		}
@@ -855,10 +969,10 @@ static void write_packet_ack_nack_desc_egprs(
 		bitvec_write_field(dest, &wp, 0, 1); // CRBB_Exist
 	}
 	LOGP(DRLCMACUL, LOGL_DEBUG,
-		"EGPRS URBB, urbb len = %d, SSN = %d, ESN_CRBB = %d, "
-		"len present = %s,desc len = %d, "
+		"EGPRS URBB, urbb len = %d, SSN = %u, ESN_CRBB = %u, "
+		"desc len = %d, "
 		"SNS = %d, WS = %d, V(Q) = %d, V(R) = %d%s%s\n",
-		urbb_len, ssn, esn_crbb, len_coded ? "yes" : "No" , len,
+		urbb_len, ssn, esn_crbb, len,
 		window->sns(), window->ws(), window->v_q(), window->v_r(),
 		bow ? ", BOW" : "", eow ? ", EOW" : "");
 
@@ -870,13 +984,13 @@ static void write_packet_ack_nack_desc_egprs(
 }
 
 static void write_packet_uplink_ack_egprs(
-	struct gprs_rlcmac_bts *bts, bitvec * dest, unsigned& wp,
+	bitvec * dest, unsigned& wp,
 	struct gprs_rlcmac_ul_tbf *tbf, bool is_final)
 {
 	bitvec_write_field(dest, &wp, 0, 2); // fixed 00
 	/* CHANNEL_CODING_COMMAND */
 	bitvec_write_field(dest, &wp,
-		tbf->current_cs().to_num() - 1, 4);
+		mcs_chan_code(tbf->current_cs()), 4);
 	/* 0: no RESEGMENT, 1: Segmentation*/
 	bitvec_write_field(dest, &wp, 1, 1);
 	bitvec_write_field(dest, &wp, 1, 1); // PRE_EMPTIVE_TRANSMISSION, TODO: This resembles GPRS, change it?
@@ -892,15 +1006,14 @@ static void write_packet_uplink_ack_egprs(
 
 	/* -2 for last bit 0 mandatory and REL5 not supported */
 	unsigned bits_ack_nack = dest->data_len * 8 - wp - 2;
-	write_packet_ack_nack_desc_egprs(bts, dest, wp, tbf->window(), is_final, bits_ack_nack);
+	write_packet_ack_nack_desc_egprs(dest, wp, tbf->window(), is_final, bits_ack_nack);
 
 	bitvec_write_field(dest, &wp, 0, 1); // fixed 0
 	bitvec_write_field(dest, &wp, 0, 1); // 0: don't have REL 5
 };
 
 void Encoding::write_packet_uplink_ack(
-	struct gprs_rlcmac_bts *bts, bitvec * dest,
-	struct gprs_rlcmac_ul_tbf *tbf, bool is_final,
+	bitvec * dest, struct gprs_rlcmac_ul_tbf *tbf, bool is_final,
 	uint8_t rrbp)
 {
 	unsigned wp = 0;
@@ -921,11 +1034,11 @@ void Encoding::write_packet_uplink_ack(
 	if (tbf->is_egprs_enabled()) {
 		/* PU_AckNack_EGPRS = on */
 		bitvec_write_field(dest, &wp, 1, 1);  // 1: EGPRS
-		write_packet_uplink_ack_egprs(bts, dest, wp, tbf, is_final);
+		write_packet_uplink_ack_egprs(dest, wp, tbf, is_final);
 	} else {
 		/* PU_AckNack_GPRS = on */
 		bitvec_write_field(dest, &wp, 0, 1);  // 0: GPRS
-		write_packet_uplink_ack_gprs(bts, dest, wp, tbf, is_final);
+		write_packet_uplink_ack_gprs(dest, wp, tbf, is_final);
 	}
 
 	LOGP(DRLCMACUL, LOGL_DEBUG,
@@ -938,19 +1051,33 @@ unsigned Encoding::write_packet_paging_request(bitvec * dest)
 {
 	unsigned wp = 0;
 
-	bitvec_write_field(dest, &wp,0x1,2);  // Payload Type
-	bitvec_write_field(dest, &wp,0x0,3);  // No polling
-	bitvec_write_field(dest, &wp,0x0,3);  // Uplink state flag
-	bitvec_write_field(dest, &wp,0x22,6);  // MESSAGE TYPE
+	bitvec_write_field(dest, &wp, 0x1, 2);  // Payload Type
+	bitvec_write_field(dest, &wp, 0x0, 3);  // No polling
+	bitvec_write_field(dest, &wp, 0x0, 3);  // Uplink state flag
+	bitvec_write_field(dest, &wp, 0x22, 6);  // MESSAGE TYPE
 
-	bitvec_write_field(dest, &wp,0x0,2);  // Page Mode
+	bitvec_write_field(dest, &wp, 0x0, 2);  // Page Mode
 
-	bitvec_write_field(dest, &wp,0x0,1);  // No PERSISTENCE_LEVEL
-	bitvec_write_field(dest, &wp,0x0,1);  // No NLN
+	bitvec_write_field(dest, &wp, 0x0, 1);  // No PERSISTENCE_LEVEL
+	bitvec_write_field(dest, &wp, 0x0, 1);  // No NLN
 
 	return wp;
 }
 
+/* 3GPP TS 44.060 § 11.2.10:
+   < Repeated Page info struct > ::=
+   { 0  -- Page request for TBF establishment
+    { 0 < PTMSI : bit (32) >
+    | 1 < Length of Mobile Identity contents : bit (4) >
+        < Mobile Identity : octet (val (Length of Mobile Identity contents)) > }
+   | 1  -- Page request for RR conn. establishment
+    { 0 < TMSI : bit (32) >
+    | 1 < Length of Mobile Identity contents : bit (4) >
+        < Mobile Identity : octet (val (Length of Mobile Identity contents)) > }
+    < CHANNEL_NEEDED : bit (2) >
+    { 0 | 1 < eMLPP_PRIORITY : bit (3) > }
+   }
+ */
 unsigned Encoding::write_repeated_page_info(bitvec * dest, unsigned& wp, uint8_t len,
 	uint8_t *identity, uint8_t chan_needed)
 {
@@ -963,7 +1090,7 @@ unsigned Encoding::write_repeated_page_info(bitvec * dest, unsigned& wp, uint8_t
 		identity++;
 		len--;
 	} else {
-		bitvec_write_field(dest, &wp,0x0,1);  // MI
+		bitvec_write_field(dest, &wp,0x1,1);  // MI
 		bitvec_write_field(dest, &wp,len,4);  // MI len
 	}
 	while (len) {
@@ -989,7 +1116,7 @@ int Encoding::rlc_write_dl_data_header(const struct gprs_rlc_data_info *rlc,
 	unsigned int bsn_delta;
 
 	switch(cs.headerTypeData()) {
-	case GprsCodingScheme::HEADER_GPRS_DATA:
+	case HEADER_GPRS_DATA:
 		gprs = static_cast<struct rlc_dl_header *>
 			((void *)data);
 
@@ -1005,7 +1132,7 @@ int Encoding::rlc_write_dl_data_header(const struct gprs_rlc_data_info *rlc,
 		gprs->bsn   = rlc->block_info[0].bsn;
 		break;
 
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_1:
+	case HEADER_EGPRS_DATA_TYPE_1:
 		egprs1 = static_cast<struct gprs_rlc_dl_header_egprs_1 *>
 			((void *)data);
 
@@ -1044,7 +1171,7 @@ int Encoding::rlc_write_dl_data_header(const struct gprs_rlc_data_info *rlc,
 		data[offs] = (data[offs] & 0b11110011) | e_fbi_header;
 		break;
 
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_2:
+	case HEADER_EGPRS_DATA_TYPE_2:
 		egprs2 = static_cast<struct gprs_rlc_dl_header_egprs_2 *>
 			((void *)data);
 
@@ -1068,7 +1195,7 @@ int Encoding::rlc_write_dl_data_header(const struct gprs_rlc_data_info *rlc,
 		data[offs] = (data[offs] & 0b11001111) | e_fbi_header;
 		break;
 
-	case GprsCodingScheme::HEADER_EGPRS_DATA_TYPE_3:
+	case HEADER_EGPRS_DATA_TYPE_3:
 		egprs3 = static_cast<struct gprs_rlc_dl_header_egprs_3 *>
 			((void *)data);
 
@@ -1098,7 +1225,7 @@ int Encoding::rlc_write_dl_data_header(const struct gprs_rlc_data_info *rlc,
 	default:
 		LOGP(DRLCMACDL, LOGL_ERROR,
 			"Encoding of uplink %s data blocks not yet supported.\n",
-			cs.name());
+			mcs_name(cs));
 		return -ENOTSUP;
 	};
 
@@ -1476,18 +1603,18 @@ Encoding::AppendResult Encoding::rlc_data_to_dl_append(
 	gprs_llc *llc, int *offset, int *num_chunks,
 	uint8_t *data_block, bool is_final, int *count_payload)
 {
-	if (cs.isGprs())
+	if (mcs_is_gprs(cs))
 		return rlc_data_to_dl_append_gprs(rdbi,
 			llc, offset, num_chunks, data_block, is_final,
 			count_payload);
 
-	if (cs.isEgprs())
+	if (mcs_is_edge(cs))
 		return rlc_data_to_dl_append_egprs(rdbi,
 			llc, offset, num_chunks, data_block, is_final,
 			count_payload);
 
 	LOGP(DRLCMACDL, LOGL_ERROR, "%s data block encoding not implemented\n",
-		cs.name());
+		mcs_name(cs));
 
 	return AR_NEED_MORE_BLOCKS;
 }
