@@ -27,7 +27,6 @@ extern "C" {
 #include <osmocom/gsm/l1sap.h>
 }
 
-struct gprs_rlcmac_bts;
 struct gprs_rlcmac_tbf;
 struct bitvec;
 struct gprs_llc;
@@ -43,14 +42,12 @@ class Encoding {
 public:
 	static int write_immediate_assignment(
 			struct gprs_rlcmac_tbf *tbf,
-			bitvec * dest, uint8_t downlink, uint16_t ra,
+			bitvec * dest, bool downlink, uint16_t ra,
 			uint32_t ref_fn, uint8_t ta, uint16_t arfcn, uint8_t ts,
-			uint8_t tsc, uint8_t usf, uint8_t polling,
+			uint8_t tsc, uint8_t usf, bool polling,
 			uint32_t fn, uint8_t alpha, uint8_t gamma,
 			int8_t ta_idx,
-			enum ph_burst_type burst_type =
-				GSM_L1_BURST_TYPE_ACCESS_0,
-			uint8_t sb = 1);
+			enum ph_burst_type burst_type);
 
 	static int write_immediate_assignment_reject(
 			bitvec *dest, uint16_t ra,
@@ -59,7 +56,6 @@ public:
 		);
 
 	static void write_packet_uplink_assignment(
-			struct gprs_rlcmac_bts *bts,
 			bitvec * dest, uint8_t old_tfi,
 			uint8_t old_downlink, uint32_t tlli, uint8_t use_tlli,
 			struct gprs_rlcmac_ul_tbf *tbf, uint8_t poll, uint8_t rrbp,
@@ -78,8 +74,7 @@ public:
 		bitvec * dest, uint32_t tlli);
 
 	static void write_packet_uplink_ack(
-			struct gprs_rlcmac_bts *bts, bitvec * dest,
-			struct gprs_rlcmac_ul_tbf *tbf, bool is_final,
+			bitvec * dest, struct gprs_rlcmac_ul_tbf *tbf, bool is_final,
 			uint8_t rrbp);
 
 	static int write_paging_request(bitvec * dest, uint8_t *ptmsi, uint16_t ptmsi_len);
