@@ -27,8 +27,11 @@
 #include <gprs_coding_scheme.h>
 #include <pdch.h>
 
-#define BSSGP_TIMER_T1	30	/* Guards the (un)blocking procedures */
-#define BSSGP_TIMER_T2	30	/* Guards the reset procedure */
+extern "C" {
+	#include <osmocom/gsm/protocol/gsm_23_003.h>
+	#include <osmocom/core/utils.h>
+	#include "coding_scheme.h"
+}
 
 /* Tuning parameters for BSSGP flow control */
 #define FC_DEFAULT_LIFE_TIME_SECS 10		/* experimental value, 10s */
@@ -47,35 +50,7 @@ extern uint16_t spoof_mcc, spoof_mnc;
 extern bool spoof_mnc_3_digits;
 
 static void bvc_timeout(void *_priv);
-
-static int parse_imsi(struct tlv_parsed *tp, char *imsi)
-{
-	uint8_t imsi_len;
-	uint8_t *bcd_imsi;
-	int i, j;
-
-	if (!TLVP_PRESENT(tp, BSSGP_IE_IMSI))
-		return -EINVAL;
-
-	imsi_len = TLVP_LEN(tp, BSSGP_IE_IMSI);
-	bcd_imsi = (uint8_t *) TLVP_VAL(tp, BSSGP_IE_IMSI);
-
-	if ((bcd_imsi[0] & 0x08))
-		imsi_len = imsi_len * 2 - 1;
-	else
-		imsi_len = (imsi_len - 1) * 2;
-	for (i = 0, j = 0; j < imsi_len && j < 15; j++)
-	{
-		if (!(j & 1)) {
-			imsi[j] = (bcd_imsi[i] >> 4) + '0';
-			i++;
-		} else
-			imsi[j] = (bcd_imsi[i] & 0xf) + '0';
-	}
-	imsi[j] = '\0';
-
-	return 0;
-}
+static int gprs_ns_reconnect(struct gprs_nsvc *nsvc);
 
 #if 0
 static int parse_ra_cap(struct tlv_parsed *tp, MS_Radio_Access_capability_t *rac)
@@ -113,9 +88,10 @@ static int gprs_bssgp_pcu_rx_dl_ud(struct msgb *msg, struct tlv_parsed *tp)
 	uint32_t tlli_old = 0;
 	uint8_t *data;
 	uint16_t len;
-	char imsi[16] = "000";
+	char imsi[OSMO_IMSI_BUF_SIZE] = "000";
 	uint8_t ms_class = 0;
 	uint8_t egprs_ms_class = 0;
+	int rc;
 #if 0
 	MS_Radio_Access_capability_t rac;
 #endif
@@ -141,7 +117,16 @@ static int gprs_bssgp_pcu_rx_dl_ud(struct msgb *msg, struct tlv_parsed *tp)
 	/* read IMSI. if no IMSI exists, use first paging block (any paging),
 	 * because during attachment the IMSI might not be known, so the MS
 	 * will listen to all paging blocks. */
-	parse_imsi(tp, imsi);
+	if (TLVP_PRESENT(tp, BSSGP_IE_IMSI))
+	{
+		/* gsm48_mi_to_string() returns number of bytes written, including '\0' */
+		rc = gsm48_mi_to_string(imsi, sizeof(imsi), TLVP_VAL(tp, BSSGP_IE_IMSI),
+							    TLVP_LEN(tp, BSSGP_IE_IMSI));
+		if (rc != GSM23003_IMSI_MAX_DIGITS + 1) {
+			LOGP(DBSSGP, LOGL_NOTICE, "Failed to parse IMSI IE (rc=%d)\n", rc);
+			return bssgp_tx_status(BSSGP_CAUSE_COND_IE_ERR, NULL, msg);
+		}
+	}
 
 #if 0 /* Do not rely on this IE. TODO: make this configurable */
 	/* parse ms radio access capability */
@@ -186,11 +171,12 @@ static int gprs_bssgp_pcu_rx_dl_ud(struct msgb *msg, struct tlv_parsed *tp)
 			ms_class, egprs_ms_class, delay_csec, data, len);
 }
 
-int gprs_bssgp_pcu_rx_paging_ps(struct msgb *msg, struct tlv_parsed *tp)
+static int gprs_bssgp_pcu_rx_paging_ps(struct msgb *msg, struct tlv_parsed *tp)
 {
 	char imsi[16];
 	uint8_t *ptmsi = (uint8_t *) TLVP_VAL(tp, BSSGP_IE_TMSI);
 	uint16_t ptmsi_len = TLVP_LEN(tp, BSSGP_IE_TMSI);
+	int rc;
 
 	LOGP(DBSSGP, LOGL_NOTICE, " P-TMSI = ");
 	for (int i = 0; i < ptmsi_len; i++)
@@ -199,12 +185,19 @@ int gprs_bssgp_pcu_rx_paging_ps(struct msgb *msg, struct tlv_parsed *tp)
 	}
 	LOGPC(DBSSGP, LOGL_NOTICE, "\n");
 
-	if (parse_imsi(tp, imsi))
-	{
+	if (!TLVP_PRESENT(tp, BSSGP_IE_IMSI)) {
 		LOGP(DBSSGP, LOGL_ERROR, "No IMSI\n");
 		return -EINVAL;
 	}
 
+	/* gsm48_mi_to_string() returns number of bytes written, including '\0' */
+	rc = gsm48_mi_to_string(imsi, sizeof(imsi), TLVP_VAL(tp, BSSGP_IE_IMSI),
+						    TLVP_LEN(tp, BSSGP_IE_IMSI));
+	if (rc != GSM23003_IMSI_MAX_DIGITS + 1) {
+		LOGP(DBSSGP, LOGL_NOTICE, "Failed to parse IMSI IE (rc=%d)\n", rc);
+		return bssgp_tx_status(BSSGP_CAUSE_COND_IE_ERR, NULL, msg);
+	}
+
 	return gprs_rlcmac_paging_request(ptmsi, ptmsi_len, imsi);
 }
 
@@ -288,7 +281,7 @@ static int gprs_bssgp_pcu_rx_sign(struct msgb *msg, struct tlv_parsed *tp, struc
 	struct bssgp_normal_hdr *bgph = (struct bssgp_normal_hdr *) msgb_bssgph(msg);
 	enum bssgp_pdu_type pdu_type = (enum bssgp_pdu_type) bgph->pdu_type;
 	int rc = 0;
-	int bvci = bctx ? bctx->bvci : -1;
+	int bvci = bctx ? bctx->bvci : msgb_bvci(msg);
 	switch (pdu_type) {
 	case BSSGP_PDUT_STATUS:
 		/* already handled in libosmogb */
@@ -391,9 +384,7 @@ static int gprs_bssgp_pcu_rcvmsg(struct msgb *msg)
 	if (rc < 0) {
 		LOGP(DBSSGP, LOGL_ERROR, "Failed to parse BSSGP %s message. Invalid message was: %s\n",
 		     bssgp_pdu_str(pdu_type), msgb_hexdump(msg));
-		if (pdu_type != BSSGP_PDUT_STATUS)
-			return bssgp_tx_status(BSSGP_CAUSE_INV_MAND_INF, NULL, msg);
-		return rc;
+		return bssgp_tx_status(BSSGP_CAUSE_INV_MAND_INF, NULL, msg);
 	}
 
 	if (pdu_type == BSSGP_PDUT_BVC_RESET) {
@@ -416,10 +407,7 @@ static int gprs_bssgp_pcu_rcvmsg(struct msgb *msg)
 	/* look-up or create the BTS context for this BVC */
 	bctx = btsctx_by_bvci_nsei(ns_bvci, msgb_nsei(msg));
 
-	if (!bctx
-	 && pdu_type != BSSGP_PDUT_BVC_RESET_ACK
-	 && pdu_type != BSSGP_PDUT_BVC_UNBLOCK_ACK
-	 && pdu_type != BSSGP_PDUT_PAGING_PS)
+	if (!bctx && ns_bvci != BVCI_SIGNALLING)
 	{
 		LOGP(DBSSGP, LOGL_NOTICE, "NSEI=%u/BVCI=%u Rejecting PDU type %s for unknown BVCI\n",
 		     nsei, ns_bvci, bssgp_pdu_str(pdu_type));
@@ -445,7 +433,7 @@ static int gprs_bssgp_pcu_rcvmsg(struct msgb *msg)
 	}
 	else
 	{
-		LOGP(DBSSGP, LOGL_DEBUG, "rx BVCI_PTP gprs_bssgp_rx_ptp\n");
+		LOGP(DBSSGP, LOGL_DEBUG, "rx BVCI_PTP=%u gprs_bssgp_rx_ptp\n", ns_bvci);
 		rc = gprs_bssgp_pcu_rx_ptp(msg, &tp, bctx);
 	}
 	return rc;
@@ -514,7 +502,7 @@ int bssgp_prim_cb(struct osmo_prim_hdr *oph, void *ctx)
 	return 0;
 }
 
-static int sgsn_ns_cb(enum gprs_ns_evt event, struct gprs_nsvc *nsvc, struct msgb *msg, uint16_t bvci)
+int gprs_bssgp_ns_cb(enum gprs_ns_evt event, struct gprs_nsvc *nsvc, struct msgb *msg, uint16_t bvci)
 {
 	int rc = 0;
 	switch (event) {
@@ -540,12 +528,20 @@ static int nsvc_signal_cb(unsigned int subsys, unsigned int signal,
 		return -EINVAL;
 
 	nssd = (struct ns_signal_data *)signal_data;
-	if (nssd->nsvc != the_pcu.nsvc) {
+	if (signal != S_SNS_CONFIGURED &&  nssd->nsvc != the_pcu.nsvc) {
 		LOGP(DPCU, LOGL_ERROR, "Signal received of unknown NSVC\n");
 		return -EINVAL;
 	}
 
 	switch (signal) {
+	case S_SNS_CONFIGURED:
+		the_pcu.bvc_sig_reset = 0;
+		the_pcu.bvc_reset = 0;
+		/* There's no NS-RESET / NS-UNBLOCK procedure on IP SNS based NS-VCs */
+		the_pcu.nsvc_unblocked = 1;
+		LOGP(DPCU, LOGL_NOTICE, "NS-VC %d is unblocked.\n", the_pcu.nsvc->nsvci);
+		bvc_timeout(NULL);
+		break;
 	case S_NS_UNBLOCK:
 		if (!the_pcu.nsvc_unblocked) {
 			the_pcu.nsvc_unblocked = 1;
@@ -598,7 +594,7 @@ static unsigned count_pdch(const struct gprs_rlcmac_bts *bts)
 
 static uint32_t gprs_bssgp_max_leak_rate(GprsCodingScheme cs, int num_pdch)
 {
-	int bytes_per_rlc_block = cs.maxDataBlockBytes() * cs.numDataBlocks();
+	int bytes_per_rlc_block = cs.maxDataBlockBytes() * num_data_blocks(cs.headerTypeData());
 
 	/* n byte payload per 20ms */
 	return bytes_per_rlc_block * (1000 / 20) * num_pdch;
@@ -713,7 +709,7 @@ static GprsCodingScheme max_coding_scheme_dl(struct gprs_rlcmac_bts *bts)
 	return GprsCodingScheme::getGprsByNum(num);
 }
 
-int gprs_bssgp_tx_fc_bvc(void)
+static int gprs_bssgp_tx_fc_bvc(void)
 {
 	struct gprs_rlcmac_bts *bts;
 	uint32_t bucket_size; /* oct */
@@ -765,7 +761,7 @@ int gprs_bssgp_tx_fc_bvc(void)
 
 		LOGP(DBSSGP, LOGL_DEBUG,
 			"Computed BVC leak rate = %d, num_pdch = %d, cs = %s\n",
-			leak_rate, num_pdch, max_cs_dl.name());
+			leak_rate, num_pdch, mcs_name(max_cs_dl));
 	};
 
 	if (ms_leak_rate == 0) {
@@ -789,7 +785,7 @@ int gprs_bssgp_tx_fc_bvc(void)
 		LOGP(DBSSGP, LOGL_DEBUG,
 			"Computed MS default leak rate = %d, ms_num_pdch = %d, "
 			"cs = %s\n",
-			ms_leak_rate, ms_num_pdch, max_cs_dl.name());
+			ms_leak_rate, ms_num_pdch, mcs_name(max_cs_dl));
 	};
 
 	/* TODO: Force leak_rate to 0 on buffer bloat */
@@ -828,10 +824,12 @@ int gprs_bssgp_tx_fc_bvc(void)
 
 static void bvc_timeout(void *_priv)
 {
+	unsigned long secs;
 	if (!the_pcu.bvc_sig_reset) {
 		LOGP(DBSSGP, LOGL_INFO, "Sending reset on BVCI 0\n");
 		bssgp_tx_bvc_reset(the_pcu.bctx, 0, BSSGP_CAUSE_OML_INTERV);
-		osmo_timer_schedule(&the_pcu.bvc_timer, BSSGP_TIMER_T2, 0);
+		secs = osmo_tdef_get(the_pcu.bts->T_defs_pcu, 2, OSMO_TDEF_S, -1);
+		osmo_timer_schedule(&the_pcu.bvc_timer, secs, 0);
 		return;
 	}
 
@@ -839,7 +837,8 @@ static void bvc_timeout(void *_priv)
 		LOGP(DBSSGP, LOGL_INFO, "Sending reset on BVCI %d\n",
 			the_pcu.bctx->bvci);
 		bssgp_tx_bvc_reset(the_pcu.bctx, the_pcu.bctx->bvci, BSSGP_CAUSE_OML_INTERV);
-		osmo_timer_schedule(&the_pcu.bvc_timer, BSSGP_TIMER_T2, 0);
+		secs = osmo_tdef_get(the_pcu.bts->T_defs_pcu, 2, OSMO_TDEF_S, -1);
+		osmo_timer_schedule(&the_pcu.bvc_timer, secs, 0);
 		return;
 	}
 
@@ -847,7 +846,8 @@ static void bvc_timeout(void *_priv)
 		LOGP(DBSSGP, LOGL_INFO, "Sending unblock on BVCI %d\n",
 			the_pcu.bctx->bvci);
 		bssgp_tx_bvc_unblock(the_pcu.bctx);
-		osmo_timer_schedule(&the_pcu.bvc_timer, BSSGP_TIMER_T1, 0);
+		secs = osmo_tdef_get(the_pcu.bts->T_defs_pcu, 1, OSMO_TDEF_S, -1);
+		osmo_timer_schedule(&the_pcu.bvc_timer, secs, 0);
 		return;
 	}
 
@@ -857,23 +857,19 @@ static void bvc_timeout(void *_priv)
 	osmo_timer_schedule(&the_pcu.bvc_timer, the_pcu.bts->fc_interval, 0);
 }
 
-int gprs_ns_reconnect(struct gprs_nsvc *nsvc)
+static int gprs_ns_reconnect(struct gprs_nsvc *nsvc)
 {
 	struct gprs_nsvc *nsvc2;
 
-	if (!bssgp_nsi) {
-		LOGP(DBSSGP, LOGL_ERROR, "NS instance does not exist\n");
-		return -EINVAL;
-	}
-
 	if (nsvc != the_pcu.nsvc) {
 		LOGP(DBSSGP, LOGL_ERROR, "NSVC is invalid\n");
 		return -EBADF;
 	}
 
-	nsvc2 = gprs_ns_nsip_connect(bssgp_nsi, &nsvc->ip.bts_addr,
-		nsvc->nsei, nsvc->nsvci);
-
+	if (the_pcu.bts->gb_dialect_sns)
+		nsvc2 = gprs_ns_nsip_connect_sns(bssgp_nsi, &nsvc->ip.bts_addr, nsvc->nsei, nsvc->nsvci);
+	else
+		nsvc2 = gprs_ns_nsip_connect(bssgp_nsi, &nsvc->ip.bts_addr, nsvc->nsei, nsvc->nsvci);
 	if (!nsvc2) {
 		LOGP(DBSSGP, LOGL_ERROR, "Failed to reconnect NSVC\n");
 		return -EIO;
@@ -898,20 +894,18 @@ struct gprs_bssgp_pcu *gprs_bssgp_create_and_connect(struct gprs_rlcmac_bts *bts
 
 	the_pcu.bts = bts;
 
-	bssgp_nsi = gprs_ns_instantiate(&sgsn_ns_cb, tall_pcu_ctx);
-	if (!bssgp_nsi) {
-		LOGP(DBSSGP, LOGL_ERROR, "Failed to create NS instance\n");
-		return NULL;
+	/* don't specify remote IP/port if SNS dialect is in use; Doing so would
+	 * issue a connect() on the socket, which prevents us to dynamically communicate
+	 * with any number of IP-SNS endpoints on the SGSN side */
+	if (!bts->gb_dialect_sns) {
+		bssgp_nsi->nsip.remote_port = sgsn_port;
+		bssgp_nsi->nsip.remote_ip = sgsn_ip;
 	}
-	gprs_ns_vty_init(bssgp_nsi);
-	bssgp_nsi->nsip.remote_port = sgsn_port;
-	bssgp_nsi->nsip.remote_ip = sgsn_ip;
 	bssgp_nsi->nsip.local_port = local_port;
 	rc = gprs_ns_nsip_listen(bssgp_nsi);
 	if (rc < 0) {
 		LOGP(DBSSGP, LOGL_ERROR, "Failed to create socket\n");
-		gprs_ns_destroy(bssgp_nsi);
-		bssgp_nsi = NULL;
+		gprs_ns_close(bssgp_nsi);
 		return NULL;
 	}
 
@@ -919,11 +913,13 @@ struct gprs_bssgp_pcu *gprs_bssgp_create_and_connect(struct gprs_rlcmac_bts *bts
 	dest.sin_port = htons(sgsn_port);
 	dest.sin_addr.s_addr = htonl(sgsn_ip);
 
-	the_pcu.nsvc = gprs_ns_nsip_connect(bssgp_nsi, &dest, nsei, nsvci);
+	if (bts->gb_dialect_sns)
+		the_pcu.nsvc = gprs_ns_nsip_connect_sns(bssgp_nsi, &dest, nsei, nsvci);
+	else
+		the_pcu.nsvc = gprs_ns_nsip_connect(bssgp_nsi, &dest, nsei, nsvci);
 	if (!the_pcu.nsvc) {
 		LOGP(DBSSGP, LOGL_ERROR, "Failed to create NSVCt\n");
-		gprs_ns_destroy(bssgp_nsi);
-		bssgp_nsi = NULL;
+		gprs_ns_close(bssgp_nsi);
 		return NULL;
 	}
 
@@ -931,8 +927,7 @@ struct gprs_bssgp_pcu *gprs_bssgp_create_and_connect(struct gprs_rlcmac_bts *bts
 	if (!the_pcu.bctx) {
 		LOGP(DBSSGP, LOGL_ERROR, "Failed to create BSSGP context\n");
 		the_pcu.nsvc = NULL;
-		gprs_ns_destroy(bssgp_nsi);
-		bssgp_nsi = NULL;
+		gprs_ns_close(bssgp_nsi);
 		return NULL;
 	}
 	the_pcu.bctx->ra_id.mcc = spoof_mcc ? : mcc;
@@ -949,8 +944,7 @@ struct gprs_bssgp_pcu *gprs_bssgp_create_and_connect(struct gprs_rlcmac_bts *bts
 
 	osmo_signal_register_handler(SS_L_NS, nsvc_signal_cb, NULL);
 
-	the_pcu.bvc_timer.cb = bvc_timeout;
-
+	osmo_timer_setup(&the_pcu.bvc_timer, bvc_timeout, NULL);
 
 	return &the_pcu;
 }
@@ -973,16 +967,11 @@ void gprs_bssgp_destroy(void)
 	the_pcu.nsvc_unblocked = 0;
 	the_pcu.bvc_sig_reset = 0;
 	the_pcu.bvc_reset = 0;
-	the_pcu.bvc_unblocked = 0;	
+	the_pcu.bvc_unblocked = 0;
 
 	gprs_ns_destroy(nsi);
 
-	/* FIXME: move this to libgb: btsctx_free() */
-	llist_del(&the_pcu.bctx->list);
-#warning "This causes ASAN to complain. It is not critical for normal operation but should be fixed nevertheless"
-#if 0
-	talloc_free(the_pcu.bctx);
-#endif
+	bssgp_bvc_ctx_free(the_pcu.bctx);
 	the_pcu.bctx = NULL;
 }
 
