@@ -23,6 +23,7 @@
 #include <gprs_coding_scheme.h>
 #include "bts.h"
 #include "tbf.h"
+#include "tbf_ul.h"
 #include "gprs_debug.h"
 #include "gprs_codel.h"
 #include "pcu_utils.h"
@@ -34,6 +35,7 @@ extern "C" {
 	#include <osmocom/core/utils.h>
 	#include <osmocom/gsm/protocol/gsm_04_08.h>
 	#include <osmocom/core/logging.h>
+	#include "coding_scheme.h"
 }
 
 #define GPRS_CODEL_SLOW_INTERVAL_MS 4000
@@ -109,7 +111,7 @@ GprsMs::GprsMs(BTS *bts, uint32_t tlli) :
 	m_reserved_ul_slots(0),
 	m_current_trx(NULL),
 	m_codel_state(NULL),
-	m_mode(GprsCodingScheme::GPRS),
+	m_mode(GPRS),
 	m_dl_ctrl_msg(0)
 {
 	int codel_interval = LLC_CODEL_USE_DEFAULT;
@@ -134,6 +136,7 @@ GprsMs::GprsMs(BTS *bts, uint32_t tlli) :
 		gprs_codel_set_interval(m_codel_state, codel_interval);
 	}
 	m_last_cs_not_low = now_msec();
+	app_info_pending = false;
 }
 
 GprsMs::~GprsMs()
@@ -212,7 +215,7 @@ void GprsMs::stop_timer()
 	unref();
 }
 
-void GprsMs::set_mode(GprsCodingScheme::Mode mode)
+void GprsMs::set_mode(enum mcs_kind mode)
 {
 	m_mode = mode;
 
@@ -220,34 +223,34 @@ void GprsMs::set_mode(GprsCodingScheme::Mode mode)
 		return;
 
 	switch (m_mode) {
-	case GprsCodingScheme::GPRS:
-		if (!m_current_cs_ul.isGprs()) {
+	case GPRS:
+		if (!mcs_is_gprs(m_current_cs_ul)) {
 			m_current_cs_ul = GprsCodingScheme::getGprsByNum(
 				m_bts->bts_data()->initial_cs_ul);
 			if (!m_current_cs_ul.isValid())
-				m_current_cs_ul = GprsCodingScheme::CS1;
+				m_current_cs_ul = CS1;
 		}
-		if (!m_current_cs_dl.isGprs()) {
+		if (!mcs_is_gprs(m_current_cs_dl)) {
 			m_current_cs_dl = GprsCodingScheme::getGprsByNum(
 				m_bts->bts_data()->initial_cs_dl);
 			if (!m_current_cs_dl.isValid())
-				m_current_cs_dl = GprsCodingScheme::CS1;
+				m_current_cs_dl = CS1;
 		}
 		break;
 
-	case GprsCodingScheme::EGPRS_GMSK:
-	case GprsCodingScheme::EGPRS:
-		if (!m_current_cs_ul.isEgprs()) {
+	case EGPRS_GMSK:
+	case EGPRS:
+		if (!mcs_is_edge(m_current_cs_ul)) {
 			m_current_cs_ul = GprsCodingScheme::getEgprsByNum(
 				m_bts->bts_data()->initial_mcs_ul);
 			if (!m_current_cs_ul.isValid())
-				m_current_cs_ul = GprsCodingScheme::MCS1;
+				m_current_cs_ul = MCS1;
 		}
-		if (!m_current_cs_dl.isEgprs()) {
+		if (!mcs_is_edge(m_current_cs_dl)) {
 			m_current_cs_dl = GprsCodingScheme::getEgprsByNum(
 				m_bts->bts_data()->initial_mcs_dl);
 			if (!m_current_cs_dl.isValid())
-				m_current_cs_dl = GprsCodingScheme::MCS1;
+				m_current_cs_dl = MCS1;
 		}
 		break;
 	}
@@ -522,12 +525,12 @@ void GprsMs::update_error_rate(gprs_rlcmac_tbf *tbf, int error_rate)
 	m_nack_rate_dl = error_rate;
 
 	if (error_rate > bts_data->cs_adj_upper_limit) {
-		if (m_current_cs_dl.to_num() > 1) {
+		if (mcs_chan_code(m_current_cs_dl) > 0) {
 			m_current_cs_dl.dec(mode());
 			LOGP(DRLCMACDL, LOGL_INFO,
 				"MS (IMSI %s): High error rate %d%%, "
 				"reducing CS level to %s\n",
-				imsi(), error_rate, m_current_cs_dl.name());
+				imsi(), error_rate, mcs_name(m_current_cs_dl));
 			m_last_cs_not_low = now;
 		}
 	} else if (error_rate < bts_data->cs_adj_lower_limit) {
@@ -539,7 +542,7 @@ void GprsMs::update_error_rate(gprs_rlcmac_tbf *tbf, int error_rate)
 				       "MS (IMSI %s): Low error rate %d%%, "
 				       "increasing DL CS level to %s\n",
 				       imsi(), error_rate,
-				       m_current_cs_dl.name());
+				       mcs_name(m_current_cs_dl));
 			       m_last_cs_not_low = now;
 		       } else {
 			       LOGP(DRLCMACDL, LOGL_DEBUG,
@@ -563,14 +566,14 @@ GprsCodingScheme GprsMs::max_cs_ul() const
 	OSMO_ASSERT(m_bts != NULL);
 	bts_data = m_bts->bts_data();
 
-	if (m_current_cs_ul.isGprs()) {
+	if (mcs_is_gprs(m_current_cs_ul)) {
 		if (!bts_data->max_cs_ul)
-			return GprsCodingScheme(GprsCodingScheme::CS4);
+			return GprsCodingScheme(CS4);
 
 		return GprsCodingScheme::getGprsByNum(bts_data->max_cs_ul);
 	}
 
-	if (!m_current_cs_ul.isEgprs())
+	if (!mcs_is_edge(m_current_cs_ul))
 		return GprsCodingScheme(); /* UNKNOWN */
 
 	if (bts_data->max_mcs_ul)
@@ -578,10 +581,10 @@ GprsCodingScheme GprsMs::max_cs_ul() const
 	else if (bts_data->max_cs_ul)
 		return GprsCodingScheme::getEgprsByNum(bts_data->max_cs_ul);
 
-	return GprsCodingScheme(GprsCodingScheme::MCS4);
+	return GprsCodingScheme(MCS4);
 }
 
-void GprsMs::set_current_cs_dl(GprsCodingScheme::Scheme scheme)
+void GprsMs::set_current_cs_dl(CodingScheme scheme)
 {
 	m_current_cs_dl = scheme;
 }
@@ -593,14 +596,14 @@ GprsCodingScheme GprsMs::max_cs_dl() const
 	OSMO_ASSERT(m_bts != NULL);
 	bts_data = m_bts->bts_data();
 
-	if (m_current_cs_dl.isGprs()) {
+	if (mcs_is_gprs(m_current_cs_dl)) {
 		if (!bts_data->max_cs_dl)
-			return GprsCodingScheme(GprsCodingScheme::CS4);
+			return GprsCodingScheme(CS4);
 
 		return GprsCodingScheme::getGprsByNum(bts_data->max_cs_dl);
 	}
 
-	if (!m_current_cs_dl.isEgprs())
+	if (!mcs_is_edge(m_current_cs_dl))
 		return GprsCodingScheme(); /* UNKNOWN */
 
 	if (bts_data->max_mcs_dl)
@@ -608,7 +611,7 @@ GprsCodingScheme GprsMs::max_cs_dl() const
 	else if (bts_data->max_cs_dl)
 		return GprsCodingScheme::getEgprsByNum(bts_data->max_cs_dl);
 
-	return GprsCodingScheme(GprsCodingScheme::MCS4);
+	return GprsCodingScheme(MCS4);
 }
 
 void GprsMs::update_cs_ul(const pcu_l1_meas *meas)
@@ -620,41 +623,54 @@ void GprsMs::update_cs_ul(const pcu_l1_meas *meas)
 	int low;
 	int high;
 	GprsCodingScheme new_cs_ul = m_current_cs_ul;
-	unsigned current_cs_num = m_current_cs_ul.to_num();
+	uint8_t current_cs = mcs_chan_code(m_current_cs_ul);
 
 	bts_data = m_bts->bts_data();
 
 	if (!max_cs_ul) {
 		LOGP(DRLCMACMEAS, LOGL_ERROR,
 			"max_cs_ul cannot be derived (current UL CS: %s)\n",
-			m_current_cs_ul.name());
+			mcs_name(m_current_cs_ul));
 		return;
 	}
 
-	OSMO_ASSERT(current_cs_num > 0);
-
-	if (!m_current_cs_ul)
+	if (!m_current_cs_ul) {
+		LOGP(DRLCMACMEAS, LOGL_ERROR,
+		     "Unable to update UL (M)CS because it's not set: %s\n",
+		     mcs_name(m_current_cs_ul));
 		return;
+	}
 
-	if (!meas->have_link_qual)
+	if (!meas->have_link_qual) {
+		LOGP(DRLCMACMEAS, LOGL_ERROR,
+		     "Unable to update UL (M)CS %s because we don't have link quality measurements.\n",
+		     mcs_name(m_current_cs_ul));
 		return;
+	}
 
-	old_link_qual = meas->link_qual;
-
-	if (m_current_cs_ul.isGprs()) {
-		low  = bts_data->cs_lqual_ranges[current_cs_num-1].low;
-		high = bts_data->cs_lqual_ranges[current_cs_num-1].high;
-	} else if (m_current_cs_ul.isEgprs()) {
-		if (current_cs_num > MAX_GPRS_CS)
-			current_cs_num = MAX_GPRS_CS;
-		low  = bts_data->mcs_lqual_ranges[current_cs_num-1].low;
-		high = bts_data->mcs_lqual_ranges[current_cs_num-1].high;
+	if (mcs_is_gprs(m_current_cs_ul)) {
+		if (current_cs >= MAX_GPRS_CS)
+			current_cs = MAX_GPRS_CS - 1;
+		low  = bts_data->cs_lqual_ranges[current_cs].low;
+		high = bts_data->cs_lqual_ranges[current_cs].high;
+	} else if (mcs_is_edge(m_current_cs_ul)) {
+		if (current_cs >= MAX_EDGE_MCS)
+			current_cs = MAX_EDGE_MCS - 1;
+		low  = bts_data->mcs_lqual_ranges[current_cs].low;
+		high = bts_data->mcs_lqual_ranges[current_cs].high;
 	} else {
+		LOGP(DRLCMACMEAS, LOGL_ERROR,
+		     "Unable to update UL (M)CS because it's neither GPRS nor EDGE: %s\n",
+		     mcs_name(m_current_cs_ul));
 		return;
 	}
 
+	/* To avoid rapid changes of the coding scheme, we also take
+	 * the old link quality value into account (if present). */
 	if (m_l1_meas.have_link_qual)
 		old_link_qual = m_l1_meas.link_qual;
+	else
+		old_link_qual = meas->link_qual;
 
 	if (meas->link_qual < low &&  old_link_qual < low)
 		new_cs_ul.dec(mode());
@@ -665,11 +681,11 @@ void GprsMs::update_cs_ul(const pcu_l1_meas *meas)
 	if (m_current_cs_ul != new_cs_ul) {
 		LOGP(DRLCMACMEAS, LOGL_INFO,
 			"MS (IMSI %s): "
-			"Link quality %ddB (%ddB) left window [%d, %d], "
+			"Link quality %ddB (old %ddB) left window [%d, %d], "
 			"modifying uplink CS level: %s -> %s\n",
 			imsi(), meas->link_qual, old_link_qual,
 			low, high,
-			m_current_cs_ul.name(), new_cs_ul.name());
+			mcs_name(m_current_cs_ul), mcs_name(new_cs_ul));
 
 		m_current_cs_ul = new_cs_ul;
 	}
@@ -733,7 +749,7 @@ GprsCodingScheme GprsMs::current_cs_dl() const
 	cs.dec(mode());
 
 	/* CS-2 doesn't gain throughput with small packets, further reduce to CS-1 */
-	if (cs == GprsCodingScheme(GprsCodingScheme::CS2))
+	if (cs == GprsCodingScheme(CS2))
 		cs.dec(mode());
 
 	return cs;
