@@ -26,6 +26,7 @@ struct gprs_codel;
 #include "cxx_linuxlist.h"
 #include "llc.h"
 #include "tbf.h"
+#include "tbf_dl.h"
 #include "pcu_l1_if.h"
 #include <gprs_coding_scheme.h>
 
@@ -74,8 +75,8 @@ public:
 	bool check_tlli(uint32_t tlli);
 
 	void reset();
-	GprsCodingScheme::Mode mode() const;
-	void set_mode(GprsCodingScheme::Mode mode);
+	enum mcs_kind mode() const;
+	void set_mode(enum mcs_kind mode);
 
 	const char *imsi() const;
 	void set_imsi(const char *imsi);
@@ -86,7 +87,7 @@ public:
 	uint8_t egprs_ms_class() const;
 	void set_ms_class(uint8_t ms_class);
 	void set_egprs_ms_class(uint8_t ms_class);
-	void set_current_cs_dl(GprsCodingScheme::Scheme scheme);
+	void set_current_cs_dl(CodingScheme scheme);
 
 	GprsCodingScheme current_cs_ul() const;
 	GprsCodingScheme current_cs_dl() const;
@@ -136,6 +137,8 @@ public:
 	/* internal use */
 	static void timeout(void *priv_);
 
+	bool app_info_pending;
+
 protected:
 	void update_status();
 	GprsMs *ref();
@@ -181,7 +184,7 @@ private:
 	gprs_rlcmac_trx *m_current_trx;
 
 	struct gprs_codel *m_codel_state;
-	GprsCodingScheme::Mode m_mode;
+	enum mcs_kind m_mode;
 
 	unsigned m_dl_ctrl_msg;
 };
@@ -237,7 +240,7 @@ inline GprsCodingScheme GprsMs::current_cs_ul() const
 	return m_current_cs_ul;
 }
 
-inline GprsCodingScheme::Mode GprsMs::mode() const
+inline enum mcs_kind GprsMs::mode() const
 {
 	return m_mode;
 }
