@@ -16,6 +16,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+extern "C" {
+#include <osmocom/gsm/gsm_utils.h>
+}
+
 inline int msecs_to_frames(int msecs) {
 	return (msecs * (1024 * 1000 / 4615)) / 1024;
 }
@@ -40,8 +44,7 @@ inline unsigned int pcu_bitcount(T x)
 	return count;
 }
 
-template <typename T>
-inline T pcu_lsb(T x)
+inline uint8_t pcu_lsb(uint8_t x)
 {
 	return x & -x;
 }
