@@ -4,6 +4,9 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <osmocom/core/tdef.h>
+#include <osmocom/core/utils.h>
+#include <osmocom/vty/tdef_vty.h>
 #include <osmocom/vty/logging.h>
 #include <osmocom/vty/stats.h>
 #include <osmocom/vty/misc.h>
@@ -26,12 +29,15 @@ static const struct value_string pcu_gsmtap_categ_names[] = {
 	{ PCU_GSMTAP_C_DL_DATA_GPRS,	"dl-data-gprs" },
 	{ PCU_GSMTAP_C_DL_DATA_EGPRS,	"dl-data-egprs" },
 	{ PCU_GSMTAP_C_DL_PTCCH,	"dl-ptcch" },
+	{ PCU_GSMTAP_C_DL_AGCH,		"dl-agch" },
+	{ PCU_GSMTAP_C_DL_PCH,		"dl-pch" },
 
 	{ PCU_GSMTAP_C_UL_UNKNOWN,	"ul-unknown" },
 	{ PCU_GSMTAP_C_UL_DUMMY,	"ul-dummy" },
 	{ PCU_GSMTAP_C_UL_CTRL,		"ul-ctrl" },
 	{ PCU_GSMTAP_C_UL_DATA_GPRS,	"ul-data-gprs" },
 	{ PCU_GSMTAP_C_UL_DATA_EGPRS,	"ul-data-egprs" },
+	{ PCU_GSMTAP_C_UL_RACH,		"ul-rach" },
 
 	{ 0, NULL }
 };
@@ -43,12 +49,15 @@ static const struct value_string pcu_gsmtap_categ_help[] = {
 	{ PCU_GSMTAP_C_DL_DATA_GPRS,	"Downlink Data Blocks (GPRS)" },
 	{ PCU_GSMTAP_C_DL_DATA_EGPRS,	"Downlink Data Blocks (EGPRS)" },
 	{ PCU_GSMTAP_C_DL_PTCCH,	"Downlink PTCCH Blocks" },
+	{ PCU_GSMTAP_C_DL_AGCH,		"Downlink AGCH Blocks" },
+	{ PCU_GSMTAP_C_DL_PCH,		"Downlink PCH Blocks" },
 
 	{ PCU_GSMTAP_C_UL_UNKNOWN,	"Unknown / Unparseable / Erroneous Downlink Blocks" },
 	{ PCU_GSMTAP_C_UL_DUMMY,	"Uplink Dummy Blocks" },
 	{ PCU_GSMTAP_C_UL_CTRL,		"Uplink Control Blocks" },
 	{ PCU_GSMTAP_C_UL_DATA_GPRS,	"Uplink Data Blocks (GPRS)" },
 	{ PCU_GSMTAP_C_UL_DATA_EGPRS,	"Uplink Data Blocks (EGPRS)" },
+	{ PCU_GSMTAP_C_UL_RACH,		"Uplink RACH Bursts" },
 
 	{ 0, NULL }
 };
@@ -251,9 +260,8 @@ static int config_write_pcu(struct vty *vty)
 		vty_out(vty, " two-phase-access%s", VTY_NEWLINE);
 	vty_out(vty, " alpha %d%s", bts->alpha, VTY_NEWLINE);
 	vty_out(vty, " gamma %d%s", bts->gamma * 2, VTY_NEWLINE);
-	if (bts->dl_tbf_idle_msec)
-		vty_out(vty, " dl-tbf-idle-time %d%s", bts->dl_tbf_idle_msec,
-			VTY_NEWLINE);
+	if (!bts->dl_tbf_preemptive_retransmission)
+		vty_out(vty, " no dl-tbf-preemptive-retransmission%s", VTY_NEWLINE);
 	if (strcmp(bts->pcu_sock_path, PCU_SOCK_DEFAULT))
 		vty_out(vty, " pcu-socket %s%s", bts->pcu_sock_path, VTY_NEWLINE);
 
@@ -265,6 +273,13 @@ static int config_write_pcu(struct vty *vty)
 		}
 	}
 
+	if (bts->gb_dialect_sns)
+		vty_out(vty, " gb-dialect ip-sns%s", VTY_NEWLINE);
+	else
+		vty_out(vty, " gb-dialect classic%s", VTY_NEWLINE);
+
+	osmo_tdef_vty_write(vty, bts->T_defs_pcu, " timer ");
+
 	return CMD_SUCCESS;
 }
 
@@ -841,56 +856,90 @@ DEFUN(show_bts_stats,
 }
 
 #define IDLE_TIME_STR "keep an idle DL TBF alive for the time given\n"
-DEFUN(cfg_pcu_dl_tbf_idle_time,
+DEFUN_DEPRECATED(cfg_pcu_dl_tbf_idle_time,
       cfg_pcu_dl_tbf_idle_time_cmd,
       "dl-tbf-idle-time <1-5000>",
       IDLE_TIME_STR "idle time in msec")
 {
+	vty_out(vty, "%% 'dl-tbf-idle-time' is now deprecated: use 'timer X2031 <val>' instead%s", VTY_NEWLINE);
+
 	struct gprs_rlcmac_bts *bts = bts_main_data();
 
-	bts->dl_tbf_idle_msec = atoi(argv[0]);
-
+	if (osmo_tdef_set(bts->T_defs_pcu, -2031, atoi(argv[0]), OSMO_TDEF_MS) < 0)
+		return CMD_WARNING;
 	return CMD_SUCCESS;
 }
 
-DEFUN(cfg_pcu_no_dl_tbf_idle_time,
+DEFUN_DEPRECATED(cfg_pcu_no_dl_tbf_idle_time,
       cfg_pcu_no_dl_tbf_idle_time_cmd,
       "no dl-tbf-idle-time",
       NO_STR IDLE_TIME_STR)
 {
+	vty_out(vty, "%% 'no dl-tbf-idle-time' is now deprecated: use 'timer X2031 0' instead%s", VTY_NEWLINE);
+
 	struct gprs_rlcmac_bts *bts = bts_main_data();
 
-	bts->dl_tbf_idle_msec = 0;
+	if (osmo_tdef_set(bts->T_defs_pcu, -2031, 0, OSMO_TDEF_MS) < 0)
+		return CMD_WARNING;
+	return CMD_SUCCESS;
+}
+
+#define RETRANSMISSION_STR "retransmit blocks even before the MS had a chance to receive them (better throughput," \
+			   " less readable traces)"
+DEFUN(cfg_pcu_dl_tbf_preemptive_retransmission,
+      cfg_pcu_dl_tbf_preemptive_retransmission_cmd,
+      "dl-tbf-preemptive-retransmission",
+      RETRANSMISSION_STR " (enabled by default)")
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+
+	bts->dl_tbf_preemptive_retransmission = true;
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_pcu_no_dl_tbf_preemptive_retransmission,
+      cfg_pcu_no_dl_tbf_preemptive_retransmission_cmd,
+      "no dl-tbf-preemptive-retransmission",
+      NO_STR RETRANSMISSION_STR)
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+
+	bts->dl_tbf_preemptive_retransmission = false;
 
 	return CMD_SUCCESS;
 }
 
 #define MS_IDLE_TIME_STR "keep an idle MS object alive for the time given\n"
-DEFUN(cfg_pcu_ms_idle_time,
+DEFUN_DEPRECATED(cfg_pcu_ms_idle_time,
       cfg_pcu_ms_idle_time_cmd,
       "ms-idle-time <1-7200>",
       MS_IDLE_TIME_STR "idle time in sec")
 {
+	vty_out(vty, "%% 'ms-idle-time' is now deprecated: use 'timer X2030 <val>' instead%s", VTY_NEWLINE);
+
 	struct gprs_rlcmac_bts *bts = bts_main_data();
 
-	bts->ms_idle_sec = atoi(argv[0]);
-
+	if (osmo_tdef_set(bts->T_defs_pcu, -2030, atoi(argv[0]), OSMO_TDEF_S) < 0)
+		return CMD_WARNING;
 	return CMD_SUCCESS;
 }
 
-DEFUN(cfg_pcu_no_ms_idle_time,
+DEFUN_DEPRECATED(cfg_pcu_no_ms_idle_time,
       cfg_pcu_no_ms_idle_time_cmd,
       "no ms-idle-time",
       NO_STR MS_IDLE_TIME_STR)
 {
+	vty_out(vty, "%% 'no ms-idle-time' is now deprecated: use 'timer X2030 0' instead%s", VTY_NEWLINE);
+
 	struct gprs_rlcmac_bts *bts = bts_main_data();
 
-	bts->ms_idle_sec = 0;
-
+	if (osmo_tdef_set(bts->T_defs_pcu, -2030, 0, OSMO_TDEF_S) < 0)
+		return CMD_WARNING;
 	return CMD_SUCCESS;
 }
 
-#define CS_ERR_LIMITS_STR "set thresholds for error rate based CS adjustment\n"
+#define CS_ERR_LIMITS_STR "set thresholds for error rate based downlink (M)CS adjustment\n"
 DEFUN(cfg_pcu_cs_err_limits,
       cfg_pcu_cs_err_limits_cmd,
       "cs threshold <0-100> <0-100>",
@@ -929,7 +978,7 @@ DEFUN(cfg_pcu_no_cs_err_limits,
 	return CMD_SUCCESS;
 }
 
-#define CS_DOWNGRADE_STR "set threshold for data size based CS downgrade\n"
+#define CS_DOWNGRADE_STR "set threshold for data size based downlink (M)CS downgrade\n"
 DEFUN(cfg_pcu_cs_downgrade_thrsh,
       cfg_pcu_cs_downgrade_thrsh_cmd,
       "cs downgrade-threshold <1-10000>",
@@ -958,7 +1007,7 @@ DEFUN(cfg_pcu_no_cs_downgrade_thrsh,
 DEFUN(cfg_pcu_cs_lqual_ranges,
       cfg_pcu_cs_lqual_ranges_cmd,
       "cs link-quality-ranges cs1 <0-35> cs2 <0-35> <0-35> cs3 <0-35> <0-35> cs4 <0-35>",
-      CS_STR "Set link quality ranges\n"
+      CS_STR "Set link quality ranges for each uplink CS\n"
       "Set quality range for CS-1 (high value only)\n"
       "CS-1 high (dB)\n"
       "Set quality range for CS-2\n"
@@ -992,7 +1041,7 @@ DEFUN(cfg_pcu_cs_lqual_ranges,
 DEFUN(cfg_pcu_mcs_lqual_ranges,
       cfg_pcu_mcs_lqual_ranges_cmd,
       "mcs link-quality-ranges mcs1 <0-35> mcs2 <0-35> <0-35> mcs3 <0-35> <0-35> mcs4 <0-35> <0-35> mcs5 <0-35> <0-35> mcs6 <0-35> <0-35> mcs7 <0-35> <0-35> mcs8 <0-35> <0-35> mcs9 <0-35>",
-      CS_STR "Set link quality ranges\n"
+      CS_STR "Set link quality ranges for each uplink MCS\n"
       "Set quality range for MCS-1 (high value only)\n"
       "MCS-1 high (dB)\n"
       "Set quality range for MCS-2\n"
@@ -1049,23 +1098,80 @@ DEFUN(cfg_pcu_sock,
 {
 	struct gprs_rlcmac_bts *bts = bts_main_data();
 
-	if (bts->pcu_sock_path) {
-		/* FIXME: close the interface? */
-		talloc_free(bts->pcu_sock_path);
-	}
-	bts->pcu_sock_path = talloc_strdup(tall_pcu_ctx, argv[0]);
-	/* FIXME: re-open the interface? */
+	if (vty->type != VTY_FILE)
+		vty_out(vty, "Changing PCU socket path at run-time has no effect%s", VTY_NEWLINE);
+
+	osmo_talloc_replace_string(tall_pcu_ctx, &bts->pcu_sock_path, argv[0]);
 
 	return CMD_SUCCESS;
 }
 
-DEFUN(show_tbf,
-      show_tbf_cmd,
-      "show tbf all",
-      SHOW_STR "information about TBFs\n" "All TBFs\n")
+DEFUN(cfg_pcu_gb_dialect,
+      cfg_pcu_gb_dialect_cmd,
+      "gb-dialect (classic|ip-sns)",
+      "Select which Gb interface dialect to use\n"
+      "Classic Gb interface with NS-{RESET,BLOCK,UNBLOCK} and static configuration\n"
+      "Modern Gb interface with IP-SNS (Sub Network Service) and dynamic configuration\n")
 {
 	struct gprs_rlcmac_bts *bts = bts_main_data();
-	return pcu_vty_show_tbf_all(vty, bts);
+
+	if (!strcmp(argv[0], "ip-sns"))
+		bts->gb_dialect_sns = true;
+	else
+		bts->gb_dialect_sns = false;
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(show_bts_timer, show_bts_timer_cmd,
+      "show bts-timer " OSMO_TDEF_VTY_ARG_T_OPTIONAL,
+      SHOW_STR "Show BTS controlled timers\n"
+      OSMO_TDEF_VTY_DOC_T)
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+	const char *T_arg = argc > 0 ? argv[0] : NULL;
+	return osmo_tdef_vty_show_cmd(vty, bts->T_defs_bts, T_arg, NULL);
+}
+
+DEFUN(show_timer, show_timer_cmd,
+      "show timer " OSMO_TDEF_VTY_ARG_T_OPTIONAL,
+      SHOW_STR "Show PCU timers\n"
+      OSMO_TDEF_VTY_DOC_T)
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+	const char *T_arg = argc > 0 ? argv[0] : NULL;
+	return osmo_tdef_vty_show_cmd(vty, bts->T_defs_pcu, T_arg, NULL);
+}
+
+DEFUN(cfg_pcu_timer, cfg_pcu_timer_cmd,
+      "timer " OSMO_TDEF_VTY_ARG_SET_OPTIONAL,
+      "Configure or show PCU timers\n"
+      OSMO_TDEF_VTY_DOC_SET)
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+	/* If any arguments are missing, redirect to 'show' */
+	if (argc < 2)
+		return show_timer(self, vty, argc, argv);
+	return osmo_tdef_vty_set_cmd(vty, bts->T_defs_pcu, argv);
+}
+
+DEFUN(show_tbf,
+      show_tbf_cmd,
+      "show tbf (all|ccch|pacch)",
+      SHOW_STR "information about TBFs\n"
+      "All TBFs\n"
+      "TBFs allocated via CCCH\n"
+      "TBFs allocated via PACCH\n")
+{
+	struct gprs_rlcmac_bts *bts = bts_main_data();
+	uint32_t flags = UINT32_MAX;
+
+	if (argv[0][0] == 'c')
+		flags = (1 << GPRS_RLCMAC_FLAG_CCCH);
+	else if (argv[0][0] == 'p')
+		flags = (1 << GPRS_RLCMAC_FLAG_PACCH);
+
+	return pcu_vty_show_tbf_all(vty, bts, flags);
 }
 
 DEFUN(show_ms_all,
@@ -1109,14 +1215,14 @@ static const char pcu_copyright[] =
 	"There is NO WARRANTY, to the extent permitted by law.\r\n";
 
 struct vty_app_info pcu_vty_info = {
-	.name		= "Osmo-PCU",
+	.name		= "OsmoPCU",
 	.version	= PACKAGE_VERSION,
 	.copyright	= pcu_copyright,
 	.go_parent_cb	= pcu_vty_go_parent,
 	.is_config_node	= pcu_vty_is_config_node,
 };
 
-int pcu_vty_init(const struct log_info *cat)
+int pcu_vty_init(void)
 {
 //	install_element_ve(&show_pcu_cmd);
 
@@ -1133,8 +1239,8 @@ int pcu_vty_init(const struct log_info *cat)
 						NO_STR "GSMTAP Category\n",
 						"\n", "", 0);
 
-	logging_vty_add_cmds(cat);
-	osmo_stats_vty_add_cmds(cat);
+	logging_vty_add_cmds();
+	osmo_stats_vty_add_cmds();
 
 	install_node(&pcu_node, config_write_pcu);
 	install_element(CONFIG_NODE, &cfg_pcu_cmd);
@@ -1184,17 +1290,23 @@ int pcu_vty_init(const struct log_info *cat)
 	install_element(PCU_NODE, &cfg_pcu_gamma_cmd);
 	install_element(PCU_NODE, &cfg_pcu_dl_tbf_idle_time_cmd);
 	install_element(PCU_NODE, &cfg_pcu_no_dl_tbf_idle_time_cmd);
+	install_element(PCU_NODE, &cfg_pcu_dl_tbf_preemptive_retransmission_cmd);
+	install_element(PCU_NODE, &cfg_pcu_no_dl_tbf_preemptive_retransmission_cmd);
 	install_element(PCU_NODE, &cfg_pcu_ms_idle_time_cmd);
 	install_element(PCU_NODE, &cfg_pcu_no_ms_idle_time_cmd);
 	install_element(PCU_NODE, &cfg_pcu_gsmtap_categ_cmd);
 	install_element(PCU_NODE, &cfg_pcu_no_gsmtap_categ_cmd);
 	install_element(PCU_NODE, &cfg_pcu_sock_cmd);
+	install_element(PCU_NODE, &cfg_pcu_gb_dialect_cmd);
+	install_element(PCU_NODE, &cfg_pcu_timer_cmd);
 
 	install_element_ve(&show_bts_stats_cmd);
 	install_element_ve(&show_tbf_cmd);
 	install_element_ve(&show_ms_all_cmd);
 	install_element_ve(&show_ms_tlli_cmd);
 	install_element_ve(&show_ms_imsi_cmd);
+	install_element_ve(&show_bts_timer_cmd);
+	install_element_ve(&show_timer_cmd);
 
 	return 0;
 }
