@@ -32,6 +32,7 @@
 #include <pcu_l1_if.h>
 #include <rlc.h>
 #include <tbf.h>
+#include <tbf_ul.h>
 #include <pdch.h>
 
 extern "C" {
@@ -41,6 +42,7 @@ extern "C" {
 	#include <osmocom/core/linuxlist.h>
 	#include <osmocom/core/utils.h>
 	#include <osmocom/vty/vty.h>
+	#include "coding_scheme.h"
 }
 
 static void tbf_print_vty_info(struct vty *vty, gprs_rlcmac_tbf *tbf)
@@ -70,7 +72,7 @@ static void tbf_print_vty_info(struct vty *vty, gprs_rlcmac_tbf *tbf)
 	}
 	if (tbf->trx != NULL)
 		vty_out(vty, " TRX_ID=%d", tbf->trx->trx_no);
-	vty_out(vty, " CS=%s", tbf->current_cs().name());
+	vty_out(vty, " CS=%s", mcs_name(tbf->current_cs()));
 
 	if (ul_tbf) {
 		gprs_rlc_ul_window *win = ul_tbf->window();
@@ -78,7 +80,7 @@ static void tbf_print_vty_info(struct vty *vty, gprs_rlcmac_tbf *tbf)
 			ul_tbf->window_size(), win->v_q(), win->v_r());
 		vty_out(vty, "%s", VTY_NEWLINE);
 		vty_out(vty, " TBF Statistics:%s", VTY_NEWLINE);
-		if(GprsCodingScheme::GPRS == tbf->ms()->mode()) {
+		if(GPRS == tbf->ms()->mode()) {
 			vty_out_rate_ctr_group(vty, " ", ul_tbf->m_ul_gprs_ctrs);
 		} else {
 			vty_out_rate_ctr_group(vty, " ", ul_tbf->m_ul_egprs_ctrs);
@@ -91,7 +93,7 @@ static void tbf_print_vty_info(struct vty *vty, gprs_rlcmac_tbf *tbf)
 			win->window_stalled() ? " STALLED" : "");
 		vty_out(vty, "%s", VTY_NEWLINE);
 		vty_out_rate_ctr_group(vty, " ", tbf->m_ctrs);
-		if(GprsCodingScheme::GPRS == tbf->ms()->mode()) {
+		if(GPRS == tbf->ms()->mode()) {
 			vty_out_rate_ctr_group(vty, " ", dl_tbf->m_dl_gprs_ctrs);
 		} else {
 			vty_out_rate_ctr_group(vty, " ", dl_tbf->m_dl_egprs_ctrs);
@@ -100,43 +102,23 @@ static void tbf_print_vty_info(struct vty *vty, gprs_rlcmac_tbf *tbf)
 	vty_out(vty, "%s%s", VTY_NEWLINE, VTY_NEWLINE);
 }
 
-int pcu_vty_show_tbf_all(struct vty *vty, struct gprs_rlcmac_bts *bts_data)
+int pcu_vty_show_tbf_all(struct vty *vty, struct gprs_rlcmac_bts *bts_data, uint32_t flags)
 {
 	BTS *bts = bts_data->bts;
-	LListHead<gprs_rlcmac_tbf> *ms_iter;
+	LListHead<gprs_rlcmac_tbf> *iter;
 
 	vty_out(vty, "UL TBFs%s", VTY_NEWLINE);
-	llist_for_each(ms_iter, &bts->ul_tbfs())
-		tbf_print_vty_info(vty, ms_iter->entry());
+	llist_for_each(iter, &bts->ul_tbfs()) {
+		if (iter->entry()->state_flags & flags)
+			tbf_print_vty_info(vty, iter->entry());
+	}
 
 	vty_out(vty, "%sDL TBFs%s", VTY_NEWLINE, VTY_NEWLINE);
-	llist_for_each(ms_iter, &bts->dl_tbfs())
-		tbf_print_vty_info(vty, ms_iter->entry());
-
-	return CMD_SUCCESS;
-}
-
-int pcu_vty_show_ms_all(struct vty *vty, struct gprs_rlcmac_bts *bts_data)
-{
-	BTS *bts = bts_data->bts;
-	LListHead<GprsMs> *ms_iter;
-
-	llist_for_each(ms_iter, &bts->ms_store().ms_list()) {
-		GprsMs *ms = ms_iter->entry();
-
-		vty_out(vty, "MS TLLI=%08x, TA=%d, CS-UL=%s, CS-DL=%s, LLC=%zd, Cl=%d, E-Cl=%d,"
-			" TBF-UL=%s, TBF-DL=%s, IMSI=%s%s",
-			ms->tlli(),
-			ms->ta(), ms->current_cs_ul().name(),
-			ms->current_cs_dl().name(),
-			ms->llc_queue()->size(),
-			ms->ms_class(),
-			ms->egprs_ms_class(),
-			ms->ul_tbf() ? ms->ul_tbf()->state_name() : "NA",
-			ms->dl_tbf() ? ms->dl_tbf()->state_name() : "NA",
-			ms->imsi(),
-			VTY_NEWLINE);
+	llist_for_each(iter, &bts->dl_tbfs()) {
+		if (iter->entry()->state_flags & flags)
+			tbf_print_vty_info(vty, iter->entry());
 	}
+
 	return CMD_SUCCESS;
 }
 
@@ -148,12 +130,11 @@ static int show_ms(struct vty *vty, GprsMs *ms)
 
 	vty_out(vty, "MS TLLI=%08x, IMSI=%s%s", ms->tlli(), ms->imsi(), VTY_NEWLINE);
 	vty_out(vty, "  Timing advance (TA):    %d%s", ms->ta(), VTY_NEWLINE);
-	vty_out(vty, "  Coding scheme uplink:   %s%s", ms->current_cs_ul().name(),
+	vty_out(vty, "  Coding scheme uplink:   %s%s", mcs_name(ms->current_cs_ul()),
 		VTY_NEWLINE);
-	vty_out(vty, "  Coding scheme downlink: %s%s", ms->current_cs_dl().name(),
+	vty_out(vty, "  Coding scheme downlink: %s%s", mcs_name(ms->current_cs_dl()),
 		VTY_NEWLINE);
-	vty_out(vty, "  Mode:                   %s%s",
-		GprsCodingScheme::modeName(ms->mode()), VTY_NEWLINE);
+	vty_out(vty, "  Mode:                   %s%s", mode_name(ms->mode()), VTY_NEWLINE);
 	vty_out(vty, "  MS class:               %d%s", ms->ms_class(), VTY_NEWLINE);
 	vty_out(vty, "  EGPRS MS class:         %d%s", ms->egprs_ms_class(), VTY_NEWLINE);
 	vty_out(vty, "  PACCH:                  ");
@@ -223,6 +204,17 @@ static int show_ms(struct vty *vty, GprsMs *ms)
 	return CMD_SUCCESS;
 }
 
+int pcu_vty_show_ms_all(struct vty *vty, struct gprs_rlcmac_bts *bts_data)
+{
+	BTS *bts = bts_data->bts;
+	LListHead<GprsMs> *ms_iter;
+
+	llist_for_each(ms_iter, &bts->ms_store().ms_list())
+		show_ms(vty, ms_iter->entry());
+
+	return CMD_SUCCESS;
+}
+
 int pcu_vty_show_ms_by_tlli(struct vty *vty, struct gprs_rlcmac_bts *bts_data,
 	uint32_t tlli)
 {
