@@ -32,6 +32,7 @@
 #include <rlc.h>
 #include <sba.h>
 #include <tbf.h>
+#include <tbf_ul.h>
 #include <cxx_linuxlist.h>
 #include <gsm_rlcmac.h>
 
@@ -43,9 +44,11 @@ extern "C" {
 	#include <osmocom/core/gsmtap.h>
 	#include <osmocom/core/logging.h>
 	#include <osmocom/core/utils.h>
+	#include "coding_scheme.h"
 }
 
 #include <errno.h>
+#include <stdlib.h>
 #include <arpa/inet.h>
 
 extern void *tall_pcu_ctx;
@@ -248,14 +251,15 @@ continue_next:
 	return msg;
 }
 
-bool gprs_rlcmac_pdch::add_paging(uint8_t chan_needed, uint8_t *identity_lv)
+bool gprs_rlcmac_pdch::add_paging(uint8_t chan_needed, const uint8_t *mi, uint8_t mi_len)
 {
 	struct gprs_rlcmac_paging *pag = talloc_zero(tall_pcu_ctx, struct gprs_rlcmac_paging);
 	if (!pag)
 		return false;
 
 	pag->chan_needed = chan_needed;
-	memcpy(pag->identity_lv, identity_lv, identity_lv[0] + 1);
+	pag->identity_lv[0] = mi_len;
+	memcpy(&pag->identity_lv[1], mi, mi_len);
 
 	llist_add(&pag->list, &paging_list);
 
@@ -372,12 +376,11 @@ void gprs_rlcmac_pdch::rcv_control_ack(Packet_Control_Acknowledgement_t *packet,
 		"at no request\n");
 }
 
-void gprs_rlcmac_pdch::rcv_control_dl_ack_nack(Packet_Downlink_Ack_Nack_t *ack_nack, uint32_t fn)
+void gprs_rlcmac_pdch::rcv_control_dl_ack_nack(Packet_Downlink_Ack_Nack_t *ack_nack, uint32_t fn, struct pcu_l1_meas *meas)
 {
 	int8_t tfi = 0; /* must be signed */
 	struct gprs_rlcmac_dl_tbf *tbf;
 	int rc;
-	struct pcu_l1_meas meas;
 	int num_blocks;
 	uint8_t bits_data[RLC_GPRS_WS/8];
 	bitvec bits;
@@ -434,16 +437,15 @@ void gprs_rlcmac_pdch::rcv_control_dl_ack_nack(Packet_Downlink_Ack_Nack_t *ack_n
 
 	/* get measurements */
 	if (tbf->ms()) {
-		get_meas(&meas, &ack_nack->Channel_Quality_Report);
-		tbf->ms()->update_l1_meas(&meas);
+		get_meas(meas, &ack_nack->Channel_Quality_Report);
+		tbf->ms()->update_l1_meas(meas);
 	}
 }
 
-void gprs_rlcmac_pdch::rcv_control_egprs_dl_ack_nack(EGPRS_PD_AckNack_t *ack_nack, uint32_t fn)
+void gprs_rlcmac_pdch::rcv_control_egprs_dl_ack_nack(EGPRS_PD_AckNack_t *ack_nack, uint32_t fn, struct pcu_l1_meas *meas)
 {
 	int8_t tfi = 0; /* must be signed */
 	struct gprs_rlcmac_dl_tbf *tbf;
-	struct pcu_l1_meas meas;
 	int rc;
 	int num_blocks;
 	uint8_t bits_data[RLC_EGPRS_MAX_WS/8];
@@ -525,13 +527,13 @@ void gprs_rlcmac_pdch::rcv_control_egprs_dl_ack_nack(EGPRS_PD_AckNack_t *ack_nac
 	if (tbf->ms()) {
 		/* TODO: Implement Measurements parsing for EGPRS */
 		/*
-		get_meas(&meas, &ack_nack->Channel_Quality_Report);
-		tbf->ms()->update_l1_meas(&meas);
+		get_meas(meas, &ack_nack->Channel_Quality_Report);
+		tbf->ms()->update_l1_meas(meas);
 		*/
 	}
 }
 
-void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request, uint32_t fn)
+void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request, uint32_t fn, struct pcu_l1_meas *meas)
 {
 	struct gprs_rlcmac_sba *sba;
 
@@ -542,7 +544,6 @@ void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request,
 		uint8_t ms_class = 0;
 		uint8_t egprs_ms_class = 0;
 		uint8_t ta = GSM48_TA_INVALID;
-		struct pcu_l1_meas meas;
 
 		GprsMs *ms = bts()->ms_by_tlli(tlli);
 		/* Keep the ms, even if it gets idle temporarily */
@@ -607,7 +608,7 @@ void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request,
 		}
 
 		/* set control ts to current MS's TS, until assignment complete */
-		LOGPTBF(ul_tbf, LOGL_DEBUG, "change control TS %d -> %d until assinment is complete.\n",
+		LOGPTBF(ul_tbf, LOGL_DEBUG, "change control TS %d -> %d until assignment is complete.\n",
 			ul_tbf->control_ts, ts_no);
 
 		ul_tbf->control_ts = ts_no;
@@ -620,8 +621,8 @@ void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request,
 
 		/* get measurements */
 		if (ul_tbf->ms()) {
-			get_meas(&meas, request);
-			ul_tbf->ms()->update_l1_meas(&meas);
+			get_meas(meas, request);
+			ul_tbf->ms()->update_l1_meas(meas);
 		}
 		return;
 	}
@@ -631,7 +632,7 @@ void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request,
 		int8_t tfi = request->ID.u.Global_TFI.u.DOWNLINK_TFI;
 		dl_tbf = bts()->dl_tbf_by_tfi(tfi, trx_no(), ts_no);
 		if (!dl_tbf) {
-			LOGP(DRLCMAC, LOGL_NOTICE, "PACKET RESSOURCE REQ unknown downlink TFI=%d\n", tfi);
+			LOGP(DRLCMAC, LOGL_NOTICE, "PACKET RESOURCE REQ unknown downlink TFI=%d\n", tfi);
 			return;
 		}
 		LOGPTBFDL(dl_tbf, LOGL_ERROR,
@@ -644,7 +645,7 @@ void gprs_rlcmac_pdch::rcv_resource_request(Packet_Resource_Request_t *request,
 		int8_t tfi = request->ID.u.Global_TFI.u.UPLINK_TFI;
 		ul_tbf = bts()->ul_tbf_by_tfi(tfi, trx_no(), ts_no);
 		if (!ul_tbf) {
-			LOGP(DRLCMAC, LOGL_NOTICE, "PACKET RESSOURCE REQ unknown uplink TFI=%d\n", tfi);
+			LOGP(DRLCMAC, LOGL_NOTICE, "PACKET RESOURCE REQ unknown uplink TFI=%d\n", tfi);
 			return;
 		}
 		LOGPTBFUL(ul_tbf, LOGL_ERROR,
@@ -679,10 +680,18 @@ void gprs_rlcmac_pdch::rcv_measurement_report(Packet_Measurement_Report_t *repor
 }
 
 /* Received Uplink RLC control block. */
-int gprs_rlcmac_pdch::rcv_control_block(
-	const uint8_t *data, uint8_t data_len, bitvec *rlc_block, uint32_t fn)
+int gprs_rlcmac_pdch::rcv_control_block(const uint8_t *data, uint8_t data_len,
+					uint32_t fn, struct pcu_l1_meas *meas, GprsCodingScheme cs)
 {
-	RlcMacUplink_t * ul_control_block = (RlcMacUplink_t *)talloc_zero(tall_pcu_ctx, RlcMacUplink_t);
+	bitvec *rlc_block;
+	RlcMacUplink_t *ul_control_block;
+	unsigned len = cs.maxBytesUL();
+
+	if (!(rlc_block = bitvec_alloc(len, tall_pcu_ctx)))
+		return -ENOMEM;
+	bitvec_unpack(rlc_block, data);
+	ul_control_block = (RlcMacUplink_t *)talloc_zero(tall_pcu_ctx, RlcMacUplink_t);
+
 	LOGP(DRLCMAC, LOGL_DEBUG, "+++++++++++++++++++++++++ RX : Uplink Control Block +++++++++++++++++++++++++\n");
 	decode_gsm_rlcmac_uplink(rlc_block, ul_control_block);
 	LOGPC(DCSN1, LOGL_NOTICE, "\n");
@@ -699,13 +708,13 @@ int gprs_rlcmac_pdch::rcv_control_block(
 		rcv_control_ack(&ul_control_block->u.Packet_Control_Acknowledgement, fn);
 		break;
 	case MT_PACKET_DOWNLINK_ACK_NACK:
-		rcv_control_dl_ack_nack(&ul_control_block->u.Packet_Downlink_Ack_Nack, fn);
+		rcv_control_dl_ack_nack(&ul_control_block->u.Packet_Downlink_Ack_Nack, fn, meas);
 		break;
 	case MT_EGPRS_PACKET_DOWNLINK_ACK_NACK:
-		rcv_control_egprs_dl_ack_nack(&ul_control_block->u.Egprs_Packet_Downlink_Ack_Nack, fn);
+		rcv_control_egprs_dl_ack_nack(&ul_control_block->u.Egprs_Packet_Downlink_Ack_Nack, fn, meas);
 		break;
 	case MT_PACKET_RESOURCE_REQUEST:
-		rcv_resource_request(&ul_control_block->u.Packet_Resource_Request, fn);
+		rcv_resource_request(&ul_control_block->u.Packet_Resource_Request, fn, meas);
 		break;
 	case MT_PACKET_MEASUREMENT_REPORT:
 		rcv_measurement_report(&ul_control_block->u.Packet_Measurement_Report, fn);
@@ -720,6 +729,7 @@ int gprs_rlcmac_pdch::rcv_control_block(
 			ul_control_block->u.MESSAGE_TYPE);
 	}
 	talloc_free(ul_control_block);
+	bitvec_free(rlc_block);
 	return 1;
 }
 
@@ -738,17 +748,17 @@ int gprs_rlcmac_pdch::rcv_block(uint8_t *data, uint8_t len, uint32_t fn,
 	bts()->rlc_ul_bytes(len);
 
 	LOGP(DRLCMACUL, LOGL_DEBUG, "Got RLC block, coding scheme: %s, "
-		"length: %d (%d))\n", cs.name(), len, cs.usedSizeUL());
+		"length: %d (%d))\n", mcs_name(cs), len, cs.usedSizeUL());
 
-	if (cs.isGprs())
+	if (mcs_is_gprs(cs))
 		return rcv_block_gprs(data, len, fn, meas, cs);
 
-	if (cs.isEgprs())
+	if (mcs_is_edge(cs))
 		return rcv_data_block(data, len, fn, meas, cs);
 
 	bts()->decode_error();
 	LOGP(DRLCMACUL, LOGL_ERROR, "Unsupported coding scheme %s\n",
-		cs.name());
+		mcs_name(cs));
 	return -EINVAL;
 }
 
@@ -764,11 +774,11 @@ int gprs_rlcmac_pdch::rcv_data_block(uint8_t *data, uint8_t data_len, uint32_t f
 	/* These are always data blocks, since EGPRS still uses CS-1 for
 	 * control blocks (see 44.060, section 10.3, 1st par.)
 	 */
-	if (cs.isEgprs()) {
+	if (mcs_is_edge(cs)) {
 		if (!bts()->bts_data()->egprs_enabled) {
 			LOGP(DRLCMACUL, LOGL_ERROR,
 				"Got %s RLC block but EGPRS is not enabled\n",
-				cs.name());
+				mcs_name(cs));
 			return -EINVAL;
 		}
 		bts()->send_gsmtap(PCU_GSMTAP_C_UL_DATA_EGPRS, true, trx_no(), ts_no, GSMTAP_CHANNEL_PDTCH, fn,
@@ -784,7 +794,7 @@ int gprs_rlcmac_pdch::rcv_data_block(uint8_t *data, uint8_t data_len, uint32_t f
 	if (rc < 0) {
 		LOGP(DRLCMACUL, LOGL_ERROR,
 			"Got %s RLC block but header parsing has failed\n",
-			cs.name());
+			mcs_name(cs));
 		bts()->decode_error();
 		return rc;
 	}
@@ -793,7 +803,7 @@ int gprs_rlcmac_pdch::rcv_data_block(uint8_t *data, uint8_t data_len, uint32_t f
 		"Got %s RLC block: "
 		"R=%d, SI=%d, TFI=%d, CPS=%d, RSB=%d, "
 		"rc=%d\n",
-		cs.name(),
+		mcs_name(cs),
 		rlc_dec.r, rlc_dec.si, rlc_dec.tfi, rlc_dec.cps, rlc_dec.rsb,
 		rc);
 
@@ -815,21 +825,14 @@ int gprs_rlcmac_pdch::rcv_block_gprs(uint8_t *data, uint8_t data_len, uint32_t f
 	struct pcu_l1_meas *meas, GprsCodingScheme cs)
 {
 	unsigned payload = data[0] >> 6;
-	bitvec *block;
 	int rc = 0;
-	unsigned len = cs.maxBytesUL();
 
 	switch (payload) {
 	case GPRS_RLCMAC_DATA_BLOCK:
 		rc = rcv_data_block(data, data_len, fn, meas, cs);
 		break;
 	case GPRS_RLCMAC_CONTROL_BLOCK:
-		block = bitvec_alloc(len, tall_pcu_ctx);
-		if (!block)
-			return -ENOMEM;
-		bitvec_unpack(block, data);
-		rc = rcv_control_block(data, data_len, block, fn);
-		bitvec_free(block);
+		rc = rcv_control_block(data, data_len, fn, meas, cs);
 		break;
 	case GPRS_RLCMAC_CONTROL_BLOCK_OPT:
 		LOGP(DRLCMAC, LOGL_NOTICE, "GPRS_RLCMAC_CONTROL_BLOCK_OPT block payload is not supported.\n");
@@ -961,3 +964,36 @@ inline gprs_rlcmac_bts *gprs_rlcmac_pdch::bts_data() const
 	return trx->bts->bts_data();
 }
 
+/* PTCCH (Packet Timing Advance Control Channel) */
+void gprs_rlcmac_pdch::init_ptcch_msg(void)
+{
+	memset(ptcch_msg, PTCCH_TAI_FREE, PTCCH_TAI_NUM);
+	memset(ptcch_msg + PTCCH_TAI_NUM, PTCCH_PADDING, 7);
+}
+
+uint8_t gprs_rlcmac_pdch::reserve_tai(uint8_t ta)
+{
+	uint8_t tai;
+
+	for (tai = 0; tai < PTCCH_TAI_NUM; tai++) {
+		if (ptcch_msg[tai] == PTCCH_TAI_FREE) {
+			ptcch_msg[tai] = ta;
+			return tai;
+		}
+	}
+
+	/* Special case: no free TAI available */
+	return PTCCH_TAI_FREE;
+}
+
+void gprs_rlcmac_pdch::release_tai(uint8_t tai)
+{
+	OSMO_ASSERT(tai < PTCCH_TAI_NUM);
+	ptcch_msg[tai] = PTCCH_TAI_FREE;
+}
+
+void gprs_rlcmac_pdch::update_ta(uint8_t tai, uint8_t ta)
+{
+	OSMO_ASSERT(tai < PTCCH_TAI_NUM);
+	ptcch_msg[tai] = ta;
+}
