@@ -21,6 +21,8 @@
 
 #include <bts.h>
 #include <tbf.h>
+#include <tbf_dl.h>
+#include <tbf_ul.h>
 #include <rlc.h>
 #include <encoding.h>
 #include <gprs_rlcmac.h>
@@ -220,8 +222,8 @@ gprs_rlcmac_tbf::gprs_rlcmac_tbf(BTS *bts_, gprs_rlcmac_tbf_direction dir) :
 	/* The classes of these members do not have proper constructors yet.
 	 * Just set them to 0 like talloc_zero did */
 	memset(&pdch, 0, sizeof(pdch));
-	memset(&T, 0, sizeof(T));
-	memset(&N, 0, sizeof(N));
+	memset(&Tarr, 0, sizeof(Tarr));
+	memset(&Narr, 0, sizeof(Narr));
 	memset(&gsm_timer, 0, sizeof(gsm_timer));
 
 	m_rlc.init();
@@ -428,7 +430,7 @@ gprs_rlcmac_ul_tbf *tbf_alloc_ul(struct gprs_rlcmac_bts *bts,
 	}
 	tbf->m_contention_resolution_done = 1;
 	TBF_SET_ASS_ON(tbf, GPRS_RLCMAC_FLAG_PACCH, false);
-	T_START(tbf, T3169, bts->t3169, 0, "allocation (UL-TBF)", true);
+	T_START(tbf, T3169, 3169, "allocation (UL-TBF)", true);
 	tbf->update_ms(tlli, GPRS_RLCMAC_UL_TBF);
 	OSMO_ASSERT(tbf->ms());
 
@@ -563,7 +565,7 @@ void gprs_rlcmac_tbf::n_reset(enum tbf_counters n)
 		return;
 	}
 
-	N[n] = 0;
+	Narr[n] = 0;
 }
 
 /* Increment counter and check for MAX value (return true if we hit it) */
@@ -577,7 +579,7 @@ bool gprs_rlcmac_tbf::n_inc(enum tbf_counters n)
 		return true;
 	}
 
-	N[n]++;
+	Narr[n]++;
 
 	switch(n) {
 	case N3101:
@@ -595,7 +597,7 @@ bool gprs_rlcmac_tbf::n_inc(enum tbf_counters n)
 		return true;
 	}
 
-	if (N[n] == chk) {
+	if (Narr[n] == chk) {
 		LOGPTBF(this, LOGL_NOTICE, "%s exceeded MAX (%u)\n",
 			get_value_string(tbf_counters_names, n), chk);
 		return true;
@@ -612,10 +614,10 @@ void gprs_rlcmac_tbf::t_stop(enum tbf_timers t, const char *reason)
 		return;
 	}
 
-	if (osmo_timer_pending(&T[t])) {
+	if (osmo_timer_pending(&Tarr[t])) {
 		LOGPTBF(this, LOGL_DEBUG, "stopping timer %s [%s]\n",
 			get_value_string(tbf_timers_names, t), reason);
-		osmo_timer_del(&T[t]);
+		osmo_timer_del(&Tarr[t]);
 	}
 }
 
@@ -625,11 +627,11 @@ bool gprs_rlcmac_tbf::timers_pending(enum tbf_timers t)
 	uint8_t i;
 
 	if (t != T_MAX)
-		return osmo_timer_pending(&T[t]);
+		return osmo_timer_pending(&Tarr[t]);
 
 	/* we don't start with T0 because it's internal timer which requires special handling */
 	for (i = T3169; i < T_MAX; i++)
-		if (osmo_timer_pending(&T[i]))
+		if (osmo_timer_pending(&Tarr[i]))
 			return true;
 
 	return false;
@@ -661,48 +663,68 @@ T_CBACK(T3191, true)
 T_CBACK(T3193, false)
 T_CBACK(T3195, true)
 
-void gprs_rlcmac_tbf::t_start(enum tbf_timers t, uint32_t sec, uint32_t microsec, const char *reason, bool force,
+void gprs_rlcmac_tbf::t_start(enum tbf_timers t, int T, const char *reason, bool force,
 			      const char *file, unsigned line)
 {
 	int current_fn = get_current_fn();
+	int sec;
+	int microsec;
+	struct osmo_tdef *tdef;
 
-	if (t >= T_MAX) {
+	if (!(tdef = osmo_tdef_get_entry(bts->bts_data()->T_defs_bts, T)))
+		tdef = osmo_tdef_get_entry(bts->bts_data()->T_defs_pcu, T);
+
+	if (t >= T_MAX || !tdef) {
 		LOGPSRC(DTBF, LOGL_ERROR, file, line, "%s attempting to start unknown timer %s [%s], cur_fn=%d\n",
 			tbf_name(this), get_value_string(tbf_timers_names, t), reason, current_fn);
 		return;
 	}
 
-	if (!force && osmo_timer_pending(&T[t]))
+	if (!force && osmo_timer_pending(&Tarr[t]))
 		return;
 
+	switch (tdef->unit) {
+	case OSMO_TDEF_MS:
+		sec = 0;
+		microsec = tdef->val * 1000;
+		break;
+	case OSMO_TDEF_S:
+		sec = tdef->val;
+		microsec = 0;
+		break;
+	default:
+		/* so far only timers using MS and S */
+		OSMO_ASSERT(false);
+	}
+
 	LOGPSRC(DTBF, LOGL_DEBUG, file, line, "%s %sstarting timer %s [%s] with %u sec. %u microsec, cur_fn=%d\n",
-	     tbf_name(this), osmo_timer_pending(&T[t]) ? "re" : "",
+	     tbf_name(this), osmo_timer_pending(&Tarr[t]) ? "re" : "",
 	     get_value_string(tbf_timers_names, t), reason, sec, microsec, current_fn);
 
-	T[t].data = this;
+	Tarr[t].data = this;
 
 	switch(t) {
 	case T0:
-		T[t].cb = tbf_timer_cb;
+		Tarr[t].cb = tbf_timer_cb;
 		break;
 	case T3169:
-		T[t].cb = cb_T3169;
+		Tarr[t].cb = cb_T3169;
 		break;
 	case T3191:
-		T[t].cb = cb_T3191;
+		Tarr[t].cb = cb_T3191;
 		break;
 	case T3193:
-		T[t].cb = cb_T3193;
+		Tarr[t].cb = cb_T3193;
 		break;
 	case T3195:
-		T[t].cb = cb_T3195;
+		Tarr[t].cb = cb_T3195;
 		break;
 	default:
 		LOGPSRC(DTBF, LOGL_ERROR, file, line, "%s attempting to set callback for unknown timer %s [%s], cur_fn=%d\n",
 		     tbf_name(this), get_value_string(tbf_timers_names, t), reason, current_fn);
 	}
 
-	osmo_timer_schedule(&T[t], sec, microsec);
+	osmo_timer_schedule(&Tarr[t], sec, microsec);
 }
 
 int gprs_rlcmac_tbf::check_polling(uint32_t fn, uint8_t ts,
@@ -790,7 +812,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 
 	if (n_inc(N3101)) {
 		TBF_SET_STATE(this, GPRS_RLCMAC_RELEASING);
-		T_START(this, T3169, bts->bts_data()->t3169, 0, "MAX N3101 reached", false);
+		T_START(this, T3169, 3169, "MAX N3101 reached", false);
 		return;
 	}
 
@@ -805,7 +827,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 			if (ul_tbf->n_inc(N3103)) {
 				bts->pkt_ul_ack_nack_poll_failed();
 				TBF_SET_STATE(ul_tbf, GPRS_RLCMAC_RELEASING);
-				T_START(ul_tbf, T3169, ul_tbf->bts->bts_data()->t3169, 0, "MAX N3103 reached", false);
+				T_START(ul_tbf, T3169, 3169, "MAX N3103 reached", false);
 				return;
 			}
 			/* reschedule UL ack */
@@ -824,7 +846,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 		bts->pua_poll_timedout();
 		if (n_inc(N3105)) {
 			TBF_SET_STATE(this, GPRS_RLCMAC_RELEASING);
-			T_START(this, T3195, bts_data()->t3195, 0, "MAX N3105 reached", true);
+			T_START(this, T3195, 3195, "MAX N3105 reached", true);
 			bts->rlc_ass_failed();
 			bts->pua_poll_failed();
 			return;
@@ -843,7 +865,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 		bts->pda_poll_timedout();
 		if (n_inc(N3105)) {
 			TBF_SET_STATE(this, GPRS_RLCMAC_RELEASING);
-			T_START(this, T3195, bts_data()->t3195, 0, "MAX N3105 reached", true);
+			T_START(this, T3195, 3195, "MAX N3105 reached", true);
 			bts->rlc_ass_failed();
 			bts->pda_poll_failed();
 			return;
@@ -869,7 +891,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 
 		if (dl_tbf->n_inc(N3105)) {
 			TBF_SET_STATE(dl_tbf, GPRS_RLCMAC_RELEASING);
-			T_START(dl_tbf, T3195, dl_tbf->bts_data()->t3195, 0, "MAX N3105 reached", true);
+			T_START(dl_tbf, T3195, 3195, "MAX N3105 reached", true);
 			bts->pkt_dl_ack_nack_poll_failed();
 			bts->rlc_ack_failed();
 			return;
@@ -880,7 +902,7 @@ void gprs_rlcmac_tbf::poll_timeout()
 			LOGPTBF(dl_tbf, LOGL_DEBUG, "Re-send dowlink assignment on PCH (IMSI=%s)\n",
 				imsi());
 			/* send immediate assignment */
-			dl_tbf->bts->snd_dl_ass(dl_tbf, 0, imsi());
+			dl_tbf->bts->snd_dl_ass(dl_tbf, false, imsi());
 			dl_tbf->m_wait_confirm = 1;
 		}
 	} else
@@ -897,7 +919,7 @@ static int setup_tbf(struct gprs_rlcmac_tbf *tbf, GprsMs *ms, int8_t use_trx, ui
 
 	bts = tbf->bts->bts_data();
 
-	if (ms->mode() == GprsCodingScheme::EGPRS)
+	if (ms->mode() == EGPRS)
 		ms_class = egprs_ms_class;
 
 	tbf->m_created_ts = time(NULL);
@@ -933,17 +955,6 @@ static int setup_tbf(struct gprs_rlcmac_tbf *tbf, GprsMs *ms, int8_t use_trx, ui
 	return 0;
 }
 
-gprs_rlcmac_ul_tbf::gprs_rlcmac_ul_tbf(BTS *bts_) :
-	gprs_rlcmac_tbf(bts_, GPRS_RLCMAC_UL_TBF),
-	m_rx_counter(0),
-	m_contention_resolution_done(0),
-	m_final_ack_sent(0),
-	m_ul_gprs_ctrs(NULL),
-	m_ul_egprs_ctrs(NULL)
-{
-	memset(&m_usf, 0, sizeof(m_usf));
-}
-
 static int ul_tbf_dtor(struct gprs_rlcmac_ul_tbf *tbf)
 {
 	tbf->~gprs_rlcmac_ul_tbf();
@@ -952,13 +963,13 @@ static int ul_tbf_dtor(struct gprs_rlcmac_ul_tbf *tbf)
 
 static void setup_egprs_mode(gprs_rlcmac_bts *bts, GprsMs *ms)
 {
-	if (GprsCodingScheme::getEgprsByNum(bts->max_mcs_ul).isEgprsGmsk() &&
-		GprsCodingScheme::getEgprsByNum(bts->max_mcs_dl).isEgprsGmsk() &&
-		ms->mode() != GprsCodingScheme::EGPRS)
+	if (mcs_is_edge_gmsk(GprsCodingScheme::getEgprsByNum(bts->max_mcs_ul)) &&
+		mcs_is_edge_gmsk(GprsCodingScheme::getEgprsByNum(bts->max_mcs_dl)) &&
+		ms->mode() != EGPRS)
 	{
-		ms->set_mode(GprsCodingScheme::EGPRS_GMSK);
+		ms->set_mode(EGPRS_GMSK);
 	} else {
-		ms->set_mode(GprsCodingScheme::EGPRS);
+		ms->set_mode(EGPRS);
 	}
 }
 
@@ -992,8 +1003,7 @@ struct gprs_rlcmac_ul_tbf *tbf_alloc_ul_tbf(struct gprs_rlcmac_bts *bts, GprsMs
 	if (egprs_ms_class > 0 && bts->egprs_enabled) {
 		tbf->enable_egprs();
 		setup_egprs_mode(bts, ms);
-		LOGPTBF(tbf, LOGL_INFO, "Enabled EGPRS, mode %s\n",
-			GprsCodingScheme::modeName(ms->mode()));
+		LOGPTBF(tbf, LOGL_INFO, "Enabled EGPRS, mode %s\n", mode_name(ms->mode()));
 	}
 
 	rc = setup_tbf(tbf, ms, use_trx, ms_class, egprs_ms_class, single_slot);
@@ -1033,19 +1043,6 @@ gprs_rlcmac_dl_tbf::BandWidth::BandWidth() :
 	timespecclear(&dl_loss_tv);
 }
 
-gprs_rlcmac_dl_tbf::gprs_rlcmac_dl_tbf(BTS *bts_) :
-	gprs_rlcmac_tbf(bts_, GPRS_RLCMAC_DL_TBF),
-	m_tx_counter(0),
-	m_wait_confirm(0),
-	m_dl_ack_requested(false),
-	m_last_dl_poll_fn(0),
-	m_last_dl_drained_fn(0),
-	m_dl_gprs_ctrs(NULL),
-	m_dl_egprs_ctrs(NULL)
-{
-	memset(&m_llc_timer, 0, sizeof(m_llc_timer));
-}
-
 static int dl_tbf_dtor(struct gprs_rlcmac_dl_tbf *tbf)
 {
 	tbf->~gprs_rlcmac_dl_tbf();
@@ -1085,8 +1082,7 @@ struct gprs_rlcmac_dl_tbf *tbf_alloc_dl_tbf(struct gprs_rlcmac_bts *bts, GprsMs
 	if (egprs_ms_class > 0 && bts->egprs_enabled) {
 		tbf->enable_egprs();
 		setup_egprs_mode(bts, ms);
-		LOGPTBF(tbf, LOGL_INFO, "Enabled EGPRS, mode %s\n",
-			GprsCodingScheme::modeName(ms->mode()));
+		LOGPTBF(tbf, LOGL_INFO, "Enabled EGPRS, mode %s\n", mode_name(ms->mode()));
 	}
 
 	rc = setup_tbf(tbf, ms, use_trx, ms_class, 0, single_slot);
@@ -1210,7 +1206,7 @@ struct msgb *gprs_rlcmac_tbf::create_dl_ass(uint32_t fn, uint8_t ts)
 
 	if (direction == GPRS_RLCMAC_DL_TBF && !is_control_ts(ts)) {
 		LOGPTBF(this, LOGL_NOTICE,
-			"Cannot poll for downlink assigment, because MS cannot reply. (TS=%d, first common TS=%d)\n",
+			"Cannot poll for downlink assignment, because MS cannot reply. (TS=%d, first common TS=%d)\n",
 			ts, first_common_ts);
 		poll_ass_dl = 0;
 	}
@@ -1270,8 +1266,7 @@ struct msgb *gprs_rlcmac_tbf::create_dl_ass(uint32_t fn, uint8_t ts)
 		msgb_free(msg);
 		return NULL;
 	}
-	bitvec_unhex(ass_vec,
-		"2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b");
+	bitvec_unhex(ass_vec, DUMMY_VEC);
 	LOGPTBF(new_dl_tbf, LOGL_INFO, "start Packet Downlink Assignment (PACCH)\n");
 	RlcMacDownlink_t * mac_control_block = (RlcMacDownlink_t *)talloc_zero(tall_pcu_ctx, RlcMacDownlink_t);
 	Encoding::write_packet_downlink_assignment(mac_control_block,
@@ -1310,8 +1305,7 @@ struct msgb *gprs_rlcmac_tbf::create_packet_access_reject()
 
 	bitvec *packet_access_rej = bitvec_alloc(23, tall_pcu_ctx);
 
-	bitvec_unhex(packet_access_rej,
-		"2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b");
+	bitvec_unhex(packet_access_rej, DUMMY_VEC);
 
 	Encoding::write_packet_access_reject(
 		packet_access_rej, tlli());
@@ -1325,7 +1319,7 @@ struct msgb *gprs_rlcmac_tbf::create_packet_access_reject()
 
 	/* Start Tmr only if it is UL TBF */
 	if (direction == GPRS_RLCMAC_UL_TBF)
-		T_START(this, T0, 0, T_REJ_PACCH_USEC, "reject (PACCH)", true);
+		T_START(this, T0, -2000, "reject (PACCH)", true);
 
 	return msg;
 
@@ -1368,9 +1362,8 @@ struct msgb *gprs_rlcmac_tbf::create_ul_ass(uint32_t fn, uint8_t ts)
 		msgb_free(msg);
 		return NULL;
 	}
-	bitvec_unhex(ass_vec,
-		"2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b");
-	Encoding::write_packet_uplink_assignment(bts_data(), ass_vec, m_tfi,
+	bitvec_unhex(ass_vec, DUMMY_VEC);
+	Encoding::write_packet_uplink_assignment(ass_vec, m_tfi,
 		(direction == GPRS_RLCMAC_DL_TBF), tlli(),
 		is_tlli_valid(), new_tbf, 1, rrbp, bts_data()->alpha,
 		bts_data()->gamma, -1, is_egprs_enabled());
