@@ -38,14 +38,12 @@ struct bssgp_bvc_ctx;
 class GprsMs;
 struct gprs_rlcmac_bts;
 
+#endif
+
 /*
  * TBF instance
  */
 
-#define T_ASS_AGCH_USEC 200000	/* waiting after IMM.ASS confirm */
-#define T_ASS_PACCH_SEC 2	/* timeout for pacch assigment */
-#define T_REJ_PACCH_USEC 2000	/* timeout for tbf reject for PRR*/
-
 enum gprs_rlcmac_tbf_state {
 	GPRS_RLCMAC_NULL = 0,	/* new created TBF */
 	GPRS_RLCMAC_ASSIGN,	/* wait for downlink assignment */
@@ -99,15 +97,6 @@ enum gprs_rlcmac_tbf_direction {
 	GPRS_RLCMAC_UL_TBF
 };
 
-enum tbf_dl_prio {
-	DL_PRIO_NONE,
-	DL_PRIO_SENT_DATA, /* the data has been sent and not (yet) nacked */
-	DL_PRIO_LOW_AGE,   /* the age has reached the first threshold */
-	DL_PRIO_NEW_DATA,  /* the data has not been sent yet or nacked */
-	DL_PRIO_HIGH_AGE,  /* the age has reached the second threshold */
-	DL_PRIO_CONTROL,   /* a control block needs to be sent */
-};
-
 enum tbf_rlc_counters {
 	TBF_CTR_RLC_NACKED,
 };
@@ -131,28 +120,7 @@ enum tbf_egprs_counters {
 	TBF_CTR_EGPRS_DL_MCS9,
 };
 
-enum tbf_gprs_ul_counters {
-        TBF_CTR_GPRS_UL_CS1,
-        TBF_CTR_GPRS_UL_CS2,
-        TBF_CTR_GPRS_UL_CS3,
-        TBF_CTR_GPRS_UL_CS4,
-};
-
-enum tbf_egprs_ul_counters {
-        TBF_CTR_EGPRS_UL_MCS1,
-        TBF_CTR_EGPRS_UL_MCS2,
-        TBF_CTR_EGPRS_UL_MCS3,
-        TBF_CTR_EGPRS_UL_MCS4,
-        TBF_CTR_EGPRS_UL_MCS5,
-        TBF_CTR_EGPRS_UL_MCS6,
-        TBF_CTR_EGPRS_UL_MCS7,
-        TBF_CTR_EGPRS_UL_MCS8,
-        TBF_CTR_EGPRS_UL_MCS9,
-};
-
 #define LOGPTBF(tbf, level, fmt, args...) LOGP(DTBF, level, "%s " fmt, tbf_name(tbf), ## args)
-#define LOGPTBFUL(tbf, level, fmt, args...) LOGP(DTBFUL, level, "%s " fmt, tbf_name(tbf), ## args)
-#define LOGPTBFDL(tbf, level, fmt, args...) LOGP(DTBFDL, level, "%s " fmt, tbf_name(tbf), ## args)
 
 enum tbf_timers {
 	/* internal assign/reject timer */
@@ -193,7 +161,7 @@ enum tbf_counters { /* TBF counters from 3GPP TS 44.060 §13.4 */
 #define GPRS_RLCMAC_FLAG_TO_DL_ASS	7
 #define GPRS_RLCMAC_FLAG_TO_MASK	0xf0 /* timeout bits */
 
-#define T_START(tbf, t, sec, usec, r, f) tbf->t_start(t, sec, usec, r, f, __FILE__, __LINE__)
+#define T_START(tbf, t, T, r, f) tbf->t_start(t, T, r, f, __FILE__, __LINE__)
 
 #define TBF_SET_STATE(t, st) do { t->set_state(st, __FILE__, __LINE__); } while(0)
 #define TBF_SET_ASS_STATE_DL(t, st) do { t->set_ass_state_dl(st, __FILE__, __LINE__); } while(0)
@@ -205,6 +173,8 @@ enum tbf_counters { /* TBF counters from 3GPP TS 44.060 §13.4 */
 #define TBF_ASS_TYPE_SET(t, kind) do { t->ass_type_mod(kind, false, __FILE__, __LINE__); } while(0)
 #define TBF_ASS_TYPE_UNSET(t, kind) do { t->ass_type_mod(kind, true, __FILE__, __LINE__); } while(0)
 
+#ifdef __cplusplus
+
 struct gprs_rlcmac_tbf {
 	gprs_rlcmac_tbf(BTS *bts_, gprs_rlcmac_tbf_direction dir);
 
@@ -250,7 +220,7 @@ struct gprs_rlcmac_tbf {
 	void stop_timers(const char *reason);
 	bool timers_pending(enum tbf_timers t);
 	void t_stop(enum tbf_timers t, const char *reason);
-	void t_start(enum tbf_timers t, uint32_t sec, uint32_t microsec, const char *reason, bool force,
+	void t_start(enum tbf_timers t, int T, const char *reason, bool force,
 		     const char *file, unsigned line);
 	int establish_dl_tbf_on_pacch();
 
@@ -370,8 +340,8 @@ private:
 	LListHead<gprs_rlcmac_tbf> m_list;
 	LListHead<gprs_rlcmac_tbf> m_ms_list;
 	bool m_egprs_enabled;
-	struct osmo_timer_list T[T_MAX];
-	uint8_t N[N_MAX];
+	struct osmo_timer_list Tarr[T_MAX];
+	uint8_t Narr[N_MAX];
 	mutable char m_name_buf[60];
 };
 
@@ -616,220 +586,12 @@ inline void gprs_rlcmac_tbf::disable_egprs()
 	m_egprs_enabled = false;
 }
 
-struct gprs_rlcmac_dl_tbf : public gprs_rlcmac_tbf {
-	gprs_rlcmac_dl_tbf(BTS *bts);
-	gprs_rlc_dl_window *window();
-	void cleanup();
-	void enable_egprs();
-	/* dispatch Unitdata.DL messages */
-	static int handle(struct gprs_rlcmac_bts *bts,
-		const uint32_t tlli, const uint32_t old_tlli,
-		const char *imsi, const uint8_t ms_class,
-		const uint8_t egprs_ms_class, const uint16_t delay_csec,
-		const uint8_t *data, const uint16_t len);
-
-	int append_data(const uint8_t ms_class,
-			const uint16_t pdu_delay_csec,
-			const uint8_t *data, const uint16_t len);
-
-	int rcvd_dl_ack(bool final, uint8_t ssn, uint8_t *rbb);
-	int rcvd_dl_ack(bool final_ack, unsigned first_bsn, struct bitvec *rbb);
-	struct msgb *create_dl_acked_block(uint32_t fn, uint8_t ts);
-	void trigger_ass(struct gprs_rlcmac_tbf *old_tbf);
-
-	bool handle_ack_nack();
-	void request_dl_ack();
-	bool need_control_ts() const;
-	bool have_data() const;
-	int frames_since_last_poll(unsigned fn) const;
-	int frames_since_last_drain(unsigned fn) const;
-	bool keep_open(unsigned fn) const;
-	int release();
-	int abort();
-	uint16_t window_size() const;
-	void set_window_size();
-	void update_coding_scheme_counter_dl(const GprsCodingScheme cs);
-
-	/* TODO: add the gettimeofday as parameter */
-	struct msgb *llc_dequeue(bssgp_bvc_ctx *bctx);
-
-	/* Please note that all variables here will be reset when changing
-	 * from WAIT RELEASE back to FLOW state (re-use of TBF).
-	 * All states that need reset must be in this struct, so this is why
-	 * variables are in both (dl and ul) structs and not outside union.
-	 */
-	int32_t m_tx_counter; /* count all transmitted blocks */
-	uint8_t m_wait_confirm; /* wait for CCCH IMM.ASS cnf */
-	bool m_dl_ack_requested;
-	int32_t m_last_dl_poll_fn;
-	int32_t m_last_dl_drained_fn;
-
-	struct BandWidth {
-		struct timespec dl_bw_tv; /* timestamp for dl bw calculation */
-		uint32_t dl_bw_octets; /* number of octets since bw_tv */
-		uint32_t dl_throughput; /* throughput to be displayed in stats */
-
-		struct timespec dl_loss_tv; /* timestamp for loss calculation */
-		uint16_t dl_loss_lost; /* sum of lost packets */
-		uint16_t dl_loss_received; /* sum of received packets */
-
-		BandWidth();
-	} m_bw;
-
-	struct rate_ctr_group *m_dl_gprs_ctrs;
-	struct rate_ctr_group *m_dl_egprs_ctrs;
-
-protected:
-	struct ana_result {
-		unsigned received_packets;
-		unsigned lost_packets;
-		unsigned received_bytes;
-		unsigned lost_bytes;
-	};
-
-	int take_next_bsn(uint32_t fn, int previous_bsn,
-		bool *may_combine);
-	bool restart_bsn_cycle();
-	int create_new_bsn(const uint32_t fn, GprsCodingScheme cs);
-	struct msgb *create_dl_acked_block(const uint32_t fn, const uint8_t ts,
-					int index, int index2 = -1);
-	int update_window(const uint8_t ssn, const uint8_t *rbb);
-	int update_window(unsigned first_bsn, const struct bitvec *rbb);
-	int maybe_start_new_window();
-	bool dl_window_stalled() const;
-	void reuse_tbf();
-	void start_llc_timer();
-	int analyse_errors(char *show_rbb, uint8_t ssn, ana_result *res);
-	void schedule_next_frame();
-
-	enum egprs_rlc_dl_reseg_bsn_state egprs_dl_get_data
-		(int bsn, uint8_t **block_data);
-	unsigned int get_egprs_dl_spb_status(int bsn);
-	enum egprs_rlcmac_dl_spb get_egprs_dl_spb(int bsn);
-
-	struct osmo_timer_list m_llc_timer;
-
-	/* Please note that all variables below will be reset when changing
-	 * from WAIT RELEASE back to FLOW state (re-use of TBF).
-	 * All states that need reset must be in this struct, so this is why
-	 * variables are in both (dl and ul) structs and not outside union.
-	 */
-	gprs_rlc_dl_window m_window;
-};
-
-struct gprs_rlcmac_ul_tbf : public gprs_rlcmac_tbf {
-	gprs_rlcmac_ul_tbf(BTS *bts);
-	gprs_rlc_ul_window *window();
-	struct msgb *create_ul_ack(uint32_t fn, uint8_t ts);
-	bool ctrl_ack_to_toggle();
-	bool handle_ctrl_ack();
-	void enable_egprs();
-	/* blocks were acked */
-	int rcv_data_block_acknowledged(
-		const struct gprs_rlc_data_info *rlc,
-		uint8_t *data, struct pcu_l1_meas *meas);
-
-
-	/* TODO: extract LLC class? */
-	int assemble_forward_llc(const gprs_rlc_data *data);
-	int snd_ul_ud();
-
-	egprs_rlc_ul_reseg_bsn_state handle_egprs_ul_spb(
-		const struct gprs_rlc_data_info *rlc,
-		struct gprs_rlc_data *block,
-		uint8_t *data, const uint8_t block_idx);
-
-	egprs_rlc_ul_reseg_bsn_state handle_egprs_ul_first_seg(
-		const struct gprs_rlc_data_info *rlc,
-		struct gprs_rlc_data *block,
-		uint8_t *data, const uint8_t block_idx);
-
-	egprs_rlc_ul_reseg_bsn_state handle_egprs_ul_second_seg(
-		const struct gprs_rlc_data_info *rlc,
-		struct gprs_rlc_data *block,
-		uint8_t *data, const uint8_t block_idx);
-
-	uint16_t window_size() const;
-	void set_window_size();
-	void update_coding_scheme_counter_ul(const GprsCodingScheme cs);
-
-	/* Please note that all variables here will be reset when changing
-	 * from WAIT RELEASE back to FLOW state (re-use of TBF).
-	 * All states that need reset must be in this struct, so this is why
-	 * variables are in both (dl and ul) structs and not outside union.
-	 */
-	int32_t m_rx_counter; /* count all received blocks */
-	uint8_t m_usf[8];	/* list USFs per PDCH (timeslot) */
-	uint8_t m_contention_resolution_done; /* set after done */
-	uint8_t m_final_ack_sent; /* set if we sent final ack */
-
-	struct rate_ctr_group *m_ul_gprs_ctrs;
-	struct rate_ctr_group *m_ul_egprs_ctrs;
-
-protected:
-	void maybe_schedule_uplink_acknack(const gprs_rlc_data_info *rlc);
-
-	/* Please note that all variables below will be reset when changing
-	 * from WAIT RELEASE back to FLOW state (re-use of TBF).
-	 * All states that need reset must be in this struct, so this is why
-	 * variables are in both (dl and ul) structs and not outside union.
-	 */
-	gprs_rlc_ul_window m_window;
-};
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-void update_tbf_ta(struct gprs_rlcmac_ul_tbf *tbf, int8_t ta_delta);
-void set_tbf_ta(struct gprs_rlcmac_ul_tbf *tbf, uint8_t ta);
-#ifdef __cplusplus
-}
-#endif
-
 inline enum gprs_rlcmac_tbf_direction reverse(enum gprs_rlcmac_tbf_direction dir)
 {
 	return (enum gprs_rlcmac_tbf_direction)
 		((int)GPRS_RLCMAC_UL_TBF - (int)dir + (int)GPRS_RLCMAC_DL_TBF);
 }
 
-inline uint16_t gprs_rlcmac_ul_tbf::window_size() const
-{
-	return m_window.ws();
-}
-
-inline uint16_t gprs_rlcmac_dl_tbf::window_size() const
-{
-	return m_window.ws();
-}
-
-inline void gprs_rlcmac_ul_tbf::enable_egprs()
-{
-	m_window.set_sns(RLC_EGPRS_SNS);
-	gprs_rlcmac_tbf::enable_egprs();
-}
-
-inline void gprs_rlcmac_dl_tbf::enable_egprs()
-{
-	m_window.set_sns(RLC_EGPRS_SNS);
-	gprs_rlcmac_tbf::enable_egprs();
-}
-
-inline gprs_rlcmac_ul_tbf *as_ul_tbf(gprs_rlcmac_tbf *tbf)
-{
-	if (tbf && tbf->direction == GPRS_RLCMAC_UL_TBF)
-		return static_cast<gprs_rlcmac_ul_tbf *>(tbf);
-	else
-		return NULL;
-}
-
-inline gprs_rlcmac_dl_tbf *as_dl_tbf(gprs_rlcmac_tbf *tbf)
-{
-	if (tbf && tbf->direction == GPRS_RLCMAC_DL_TBF)
-		return static_cast<gprs_rlcmac_dl_tbf *>(tbf);
-	else
-		return NULL;
-}
-
 uint16_t egprs_window_size(const struct gprs_rlcmac_bts *bts_data, uint8_t slots);
 
 #endif
