@@ -21,6 +21,7 @@
 
 #include <bts.h>
 #include <tbf.h>
+#include <tbf_ul.h>
 #include <rlc.h>
 #include <encoding.h>
 #include <gprs_rlcmac.h>
@@ -43,6 +44,7 @@ extern "C" {
 	#include <osmocom/gprs/gprs_bssgp_bss.h>
 	#include <osmocom/gprs/protocol/gsm_08_18.h>
 	#include <osmocom/gsm/tlv.h>
+	#include "coding_scheme.h"
 }
 
 #include <errno.h>
@@ -53,6 +55,17 @@ extern "C" {
 
 extern void *tall_pcu_ctx;
 
+gprs_rlcmac_ul_tbf::gprs_rlcmac_ul_tbf(BTS *bts_) :
+	gprs_rlcmac_tbf(bts_, GPRS_RLCMAC_UL_TBF),
+	m_rx_counter(0),
+	m_contention_resolution_done(0),
+	m_final_ack_sent(0),
+	m_ul_gprs_ctrs(NULL),
+	m_ul_egprs_ctrs(NULL)
+{
+	memset(&m_usf, 0, sizeof(m_usf));
+}
+
 /*
  * Store received block data in LLC message(s) and forward to SGSN
  * if complete.
@@ -151,9 +164,8 @@ struct msgb *gprs_rlcmac_ul_tbf::create_ul_ack(uint32_t fn, uint8_t ts)
 		msgb_free(msg);
 		return NULL;
 	}
-	bitvec_unhex(ack_vec,
-		"2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b2b");
-	Encoding::write_packet_uplink_ack(bts_data(), ack_vec, this, final, rrbp);
+	bitvec_unhex(ack_vec, DUMMY_VEC);
+	Encoding::write_packet_uplink_ack(ack_vec, this, final, rrbp);
 	bitvec_pack(ack_vec, msgb_put(msg, 23));
 	bitvec_free(ack_vec);
 
@@ -197,7 +209,7 @@ int gprs_rlcmac_ul_tbf::rcv_data_block_acknowledged(
 	unsigned int block_idx;
 
 	/* restart T3169 */
-	T_START(this, T3169, bts_data()->t3169, 0, "acked (data)", true);
+	T_START(this, T3169, 3169, "acked (data)", true);
 
 	/* Increment RX-counter */
 	this->m_rx_counter++;
@@ -213,7 +225,7 @@ int gprs_rlcmac_ul_tbf::rcv_data_block_acknowledged(
 
 		LOGPTBFUL(this, LOGL_DEBUG,
 			  "Got %s RLC data block: CV=%d, BSN=%d, SPB=%d, PI=%d, E=%d, TI=%d, bitoffs=%d\n",
-			  rlc->cs.name(),
+			  mcs_name(rlc->cs),
 			  rdbi->cv, rdbi->bsn, rdbi->spb,
 			  rdbi->pi, rdbi->e, rdbi->ti,
 			  rlc->data_offs_bits[block_idx]);
@@ -279,7 +291,7 @@ int gprs_rlcmac_ul_tbf::rcv_data_block_acknowledged(
 				bts->decode_error();
 				LOGPTBFUL(this, LOGL_NOTICE,
 					  "Failed to decode TLLI of %s UL DATA TFI=%d.\n",
-					  rlc->cs.name(), rlc->tfi);
+					  mcs_name(rlc->cs), rlc->tfi);
 				m_window.invalidate_bsn(rdbi->bsn);
 				continue;
 			}
@@ -395,7 +407,7 @@ int gprs_rlcmac_ul_tbf::snd_ul_ud()
 		m_llc.reset_frame_space();
 		return -EIO;
 	}
-	
+
 	llc_pdu = msgb_alloc_headroom(msg_len, msg_len,"llc_pdu");
 	uint8_t *buf = msgb_push(llc_pdu, TL16V_GROSS_LEN(sizeof(uint8_t)*m_llc.frame_length()));
 	tl16v_put(buf, BSSGP_IE_LLC_PDU, sizeof(uint8_t)*m_llc.frame_length(), m_llc.frame);
@@ -480,7 +492,7 @@ egprs_rlc_ul_reseg_bsn_state gprs_rlcmac_ul_tbf::handle_egprs_ul_spb(
 
 	LOGPTBFUL(this, LOGL_DEBUG,
 		  "Got SPB(%d) cs(%s) data block with BSN (%d), TFI(%d).\n",
-		  rdbi->spb,  rlc->cs.name(), rdbi->bsn, rlc->tfi);
+		  rdbi->spb,  mcs_name(rlc->cs), rdbi->bsn, rlc->tfi);
 
 	egprs_rlc_ul_reseg_bsn_state assemble_status = EGPRS_RESEG_INVALID;
 
@@ -502,99 +514,95 @@ egprs_rlc_ul_reseg_bsn_state gprs_rlcmac_ul_tbf::handle_egprs_ul_spb(
 	 * upgrade the MCS to the type 2
 	 */
 	if (assemble_status == EGPRS_RESEG_DEFAULT) {
-		switch (GprsCodingScheme::Scheme(rlc->cs)) {
-		case GprsCodingScheme::MCS3 :
-			block->cs_last = GprsCodingScheme::MCS6;
+		switch (CodingScheme(rlc->cs)) {
+		case MCS3 :
+			block->cs_last = MCS6;
 			LOGPTBFUL(this, LOGL_DEBUG, "Upgrading to MCS6\n");
 			break;
-		case GprsCodingScheme::MCS2 :
-			block->cs_last = GprsCodingScheme::MCS5;
+		case MCS2 :
+			block->cs_last = MCS5;
 			LOGPTBFUL(this, LOGL_DEBUG, "Upgrading to MCS5\n");
 			break;
-		case GprsCodingScheme::MCS1 :
+		case MCS1 :
 			LOGPTBFUL(this, LOGL_DEBUG, "Upgrading to MCS4\n");
-			block->cs_last = GprsCodingScheme::MCS4;
+			block->cs_last = MCS4;
 			break;
 		default:
 			LOGPTBFUL(this, LOGL_ERROR,
 				  "cs(%s) Error in Upgrading to higher MCS\n",
-				  rlc->cs.name());
+				  mcs_name(rlc->cs));
 			break;
 		}
 	}
 	return assemble_status;
 }
 
-void gprs_rlcmac_ul_tbf::update_coding_scheme_counter_ul(const GprsCodingScheme cs)
+void gprs_rlcmac_ul_tbf::update_coding_scheme_counter_ul(enum CodingScheme cs)
 {
-	uint8_t coding_scheme = 0;
-
-	coding_scheme = GprsCodingScheme::Scheme(cs);
-	if (cs.isGprs()) {
-		switch (coding_scheme) {
-		case GprsCodingScheme::CS1 :
-			bts->gprs_ul_cs1();
-			rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS1]);
-			break;
-		case GprsCodingScheme::CS2 :
-			bts->gprs_ul_cs2();
-			rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS2]);
-			break;
-		case GprsCodingScheme::CS3 :
-			bts->gprs_ul_cs3();
-			rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS3]);
-			break;
-		case GprsCodingScheme::CS4 :
-			bts->gprs_ul_cs4();
-			rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS4]);
-			break;
-		}
-	} else {
-		switch (coding_scheme) {
-		case GprsCodingScheme::MCS1 :
-			bts->egprs_ul_mcs1();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS1]);
-			break;
-		case GprsCodingScheme::MCS2 :
-			bts->egprs_ul_mcs2();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS2]);
-			break;
-		case GprsCodingScheme::MCS3 :
-			bts->egprs_ul_mcs3();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS3]);
-			break;
-		case GprsCodingScheme::MCS4 :
-			bts->egprs_ul_mcs4();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS4]);
-			break;
-		case GprsCodingScheme::MCS5 :
-			bts->egprs_ul_mcs5();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS5]);
-			break;
-		case GprsCodingScheme::MCS6 :
-			bts->egprs_ul_mcs6();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS6]);
-			break;
-		case GprsCodingScheme::MCS7 :
-			bts->egprs_ul_mcs7();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS7]);
-			break;
-		case GprsCodingScheme::MCS8 :
-			bts->egprs_ul_mcs8();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS8]);
-			break;
-		case GprsCodingScheme::MCS9 :
-			bts->egprs_ul_mcs9();
-			rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS9]);
-			break;
-		}
+	switch (cs) {
+	case CS1:
+		bts->gprs_ul_cs1();
+		rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS1]);
+		break;
+	case CS2:
+		bts->gprs_ul_cs2();
+		rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS2]);
+		break;
+	case CS3:
+		bts->gprs_ul_cs3();
+		rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS3]);
+		break;
+	case CS4:
+		bts->gprs_ul_cs4();
+		rate_ctr_inc(&m_ul_gprs_ctrs->ctr[TBF_CTR_GPRS_UL_CS4]);
+		break;
+	case MCS1:
+		bts->egprs_ul_mcs1();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS1]);
+		break;
+	case MCS2:
+		bts->egprs_ul_mcs2();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS2]);
+		break;
+	case MCS3:
+		bts->egprs_ul_mcs3();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS3]);
+		break;
+	case MCS4:
+		bts->egprs_ul_mcs4();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS4]);
+		break;
+	case MCS5:
+		bts->egprs_ul_mcs5();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS5]);
+		break;
+	case MCS6:
+		bts->egprs_ul_mcs6();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS6]);
+		break;
+	case MCS7:
+		bts->egprs_ul_mcs7();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS7]);
+		break;
+	case MCS8:
+		bts->egprs_ul_mcs8();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS8]);
+		break;
+	case MCS9:
+		bts->egprs_ul_mcs9();
+		rate_ctr_inc(&m_ul_egprs_ctrs->ctr[TBF_CTR_EGPRS_UL_MCS9]);
+		break;
+	default:
+		LOGPTBFUL(this, LOGL_ERROR, "attempting to update rate counters for unsupported (M)CS %s\n",
+			  mcs_name(cs));
 	}
 }
 
 void gprs_rlcmac_ul_tbf::set_window_size()
 {
-	uint16_t ws = egprs_window_size(bts->bts_data(), ul_slots());
+	const struct gprs_rlcmac_bts *b = bts->bts_data();
+	uint16_t ws = egprs_window_size(b, ul_slots());
 	LOGPTBFUL(this, LOGL_INFO, "setting EGPRS UL window size to %u, base(%u) slots(%u) ws_pdch(%u)\n",
-		  ws, bts->bts_data()->ws_base, pcu_bitcount(ul_slots()), bts->bts_data()->ws_pdch);
+		  ws, b->ws_base, pcu_bitcount(ul_slots()), b->ws_pdch);
 	m_window.set_ws(ws);
 }
