@@ -48,7 +48,7 @@ void *tall_pcu_ctx;
 int16_t spoof_mnc = 0, spoof_mcc = 0;
 bool spoof_mnc_3_digits = false;
 
-static void check_coding_scheme(GprsCodingScheme& cs, GprsCodingScheme::Mode mode)
+static void check_coding_scheme(GprsCodingScheme& cs, enum mcs_kind mode)
 {
 	volatile unsigned expected_size;
 	bool need_padding;
@@ -59,20 +59,20 @@ static void check_coding_scheme(GprsCodingScheme& cs, GprsCodingScheme::Mode mod
 
 	/* Check static getBySizeUL() */
 	expected_size = cs.usedSizeUL();
-	if (cs.spareBitsUL() > 0 && cs.isGprs())
+	if (cs.spareBitsUL() > 0 && mcs_is_gprs(cs))
 		expected_size += 1;
 	OSMO_ASSERT(expected_size == cs.sizeUL());
 	OSMO_ASSERT(cs == GprsCodingScheme::getBySizeUL(expected_size));
 
 	/* Check static sizeUL() */
 	expected_size = cs.usedSizeDL();
-	if (cs.spareBitsDL() > 0 && cs.isGprs())
+	if (cs.spareBitsDL() > 0 && mcs_is_gprs(cs))
 		expected_size += 1;
 	OSMO_ASSERT(expected_size == cs.sizeDL());
 
 	/* Check data block sizes */
-	OSMO_ASSERT(cs.maxDataBlockBytes() * cs.numDataBlocks() < cs.maxBytesDL());
-	OSMO_ASSERT(cs.maxDataBlockBytes() * cs.numDataBlocks() < cs.maxBytesUL());
+	OSMO_ASSERT(cs.maxDataBlockBytes() * num_data_blocks(cs.headerTypeData()) < cs.maxBytesDL());
+	OSMO_ASSERT(cs.maxDataBlockBytes() * num_data_blocks(cs.headerTypeData()) < cs.maxBytesUL());
 
 	/* Check inc/dec */
 	new_cs = cs;
@@ -105,66 +105,72 @@ static void check_coding_scheme(GprsCodingScheme& cs, GprsCodingScheme::Mode mod
 
 }
 
+static bool check_strong_monotonicity(const GprsCodingScheme *cs, uint8_t last_UL, uint8_t last_DL)
+{
+	if (cs->maxBytesUL() <= last_UL)
+		return false;
+
+	if (cs->maxBytesDL() <= last_DL)
+		return false;
+
+	return true;
+}
+
 static void test_coding_scheme()
 {
 	unsigned i;
-	unsigned last_size_UL;
-	unsigned last_size_DL;
-	GprsCodingScheme::Scheme gprs_schemes[] = {
-		GprsCodingScheme::CS1,
-		GprsCodingScheme::CS2,
-		GprsCodingScheme::CS3,
-		GprsCodingScheme::CS4
+	uint8_t last_size_UL;
+	uint8_t last_size_DL;
+	CodingScheme gprs_schemes[] = {
+		CS1,
+		CS2,
+		CS3,
+		CS4
 	};
 	struct {
-		GprsCodingScheme::Scheme s;
+		CodingScheme s;
 		bool is_gmsk;
 	} egprs_schemes[] = {
-		{GprsCodingScheme::MCS1, true},
-		{GprsCodingScheme::MCS2, true},
-		{GprsCodingScheme::MCS3, true},
-		{GprsCodingScheme::MCS4, true},
-		{GprsCodingScheme::MCS5, false},
-		{GprsCodingScheme::MCS6, false},
-		{GprsCodingScheme::MCS7, false},
-		{GprsCodingScheme::MCS8, false},
-		{GprsCodingScheme::MCS9, false},
+		{ MCS1, true},
+		{ MCS2, true},
+		{ MCS3, true},
+		{ MCS4, true},
+		{ MCS5, false},
+		{ MCS6, false},
+		{ MCS7, false},
+		{ MCS8, false},
+		{ MCS9, false},
 	};
 
 	printf("=== start %s ===\n", __func__);
 
 	GprsCodingScheme cs;
 	OSMO_ASSERT(!cs);
-	OSMO_ASSERT(GprsCodingScheme::Scheme(cs) == GprsCodingScheme::UNKNOWN);
-	OSMO_ASSERT(cs == GprsCodingScheme(GprsCodingScheme::UNKNOWN));
-	OSMO_ASSERT(!cs.isCompatible(GprsCodingScheme::GPRS));
-	OSMO_ASSERT(!cs.isCompatible(GprsCodingScheme::EGPRS_GMSK));
-	OSMO_ASSERT(!cs.isCompatible(GprsCodingScheme::EGPRS));
+	OSMO_ASSERT(CodingScheme(cs) == UNKNOWN);
+	OSMO_ASSERT(cs == GprsCodingScheme(UNKNOWN));
+	OSMO_ASSERT(!cs.isCompatible(GPRS));
+	OSMO_ASSERT(!cs.isCompatible(EGPRS_GMSK));
+	OSMO_ASSERT(!cs.isCompatible(EGPRS));
 
 	last_size_UL = 0;
 	last_size_DL = 0;
 
 	for (i = 0; i < ARRAY_SIZE(gprs_schemes); i++) {
 		GprsCodingScheme current_cs(gprs_schemes[i]);
-		OSMO_ASSERT(current_cs.isGprs());
-		OSMO_ASSERT(!current_cs.isEgprs());
-		OSMO_ASSERT(!current_cs.isEgprsGmsk());
-		OSMO_ASSERT(GprsCodingScheme::Scheme(current_cs) == gprs_schemes[i]);
+		OSMO_ASSERT(mcs_is_gprs(current_cs));
+		OSMO_ASSERT(!mcs_is_edge(current_cs));
+		OSMO_ASSERT(!mcs_is_edge_gmsk(current_cs));
+		OSMO_ASSERT(CodingScheme(current_cs) == gprs_schemes[i]);
 		OSMO_ASSERT(current_cs == GprsCodingScheme(gprs_schemes[i]));
 
-		/* Check strong monotonicity */
-		OSMO_ASSERT(current_cs.maxBytesUL() > last_size_UL);
-		OSMO_ASSERT(current_cs.maxBytesDL() > last_size_DL);
+		OSMO_ASSERT(check_strong_monotonicity(&current_cs, last_size_UL, last_size_DL));
 		last_size_UL = current_cs.maxBytesUL();
 		last_size_DL = current_cs.maxBytesDL();
 
 		/* Check header types */
-		OSMO_ASSERT(current_cs.headerTypeData() ==
-			GprsCodingScheme::HEADER_GPRS_DATA);
-		OSMO_ASSERT(current_cs.headerTypeControl() ==
-			GprsCodingScheme::HEADER_GPRS_CONTROL);
+		OSMO_ASSERT(current_cs.headerTypeData() == HEADER_GPRS_DATA);
 
-		check_coding_scheme(current_cs, GprsCodingScheme::GPRS);
+		check_coding_scheme(current_cs, GPRS);
 	}
 	OSMO_ASSERT(i == 4);
 
@@ -173,21 +179,19 @@ static void test_coding_scheme()
 
 	for (i = 0; i < ARRAY_SIZE(egprs_schemes); i++) {
 		GprsCodingScheme current_cs(egprs_schemes[i].s);
-		OSMO_ASSERT(!current_cs.isGprs());
-		OSMO_ASSERT(current_cs.isEgprs());
-		OSMO_ASSERT(!!current_cs.isEgprsGmsk() == !!egprs_schemes[i].is_gmsk);
-		OSMO_ASSERT(GprsCodingScheme::Scheme(current_cs) == egprs_schemes[i].s);
+		OSMO_ASSERT(!mcs_is_gprs(current_cs));
+		OSMO_ASSERT(mcs_is_edge(current_cs));
+		OSMO_ASSERT(mcs_is_edge_gmsk(current_cs) == !!egprs_schemes[i].is_gmsk);
+		OSMO_ASSERT(CodingScheme(current_cs) == egprs_schemes[i].s);
 		OSMO_ASSERT(current_cs == GprsCodingScheme(egprs_schemes[i].s));
 
-		/* Check strong monotonicity */
-		OSMO_ASSERT(current_cs.maxBytesUL() > last_size_UL);
-		OSMO_ASSERT(current_cs.maxBytesDL() > last_size_DL);
+		OSMO_ASSERT(check_strong_monotonicity(&current_cs, last_size_UL, last_size_DL));
 		last_size_UL = current_cs.maxBytesUL();
 		last_size_DL = current_cs.maxBytesDL();
 
 		if (egprs_schemes[i].is_gmsk)
-			check_coding_scheme(current_cs, GprsCodingScheme::EGPRS_GMSK);
-		check_coding_scheme(current_cs, GprsCodingScheme::EGPRS);
+			check_coding_scheme(current_cs, EGPRS_GMSK);
+		check_coding_scheme(current_cs, EGPRS);
 	}
 	OSMO_ASSERT(i == 9);
 
@@ -208,7 +212,7 @@ static void test_rlc_unit_decoder()
 	printf("=== start %s ===\n", __func__);
 
 	/* TS 44.060, B.1 */
-	cs = GprsCodingScheme::CS4;
+	cs = CS4;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 0;
@@ -232,7 +236,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(!chunks[2].is_complete);
 
 	/* TS 44.060, B.2 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 0;
@@ -266,7 +270,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(!chunks[1].is_complete);
 
 	/* TS 44.060, B.3 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 0;
@@ -287,7 +291,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(chunks[1].is_complete);
 
 	/* TS 44.060, B.4 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 1;
 	rdbi.ti = 0;
@@ -303,7 +307,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(!chunks[0].is_complete);
 
 	/* TS 44.060, B.6 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 1;
 	rdbi.ti = 0;
@@ -319,7 +323,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(chunks[0].is_complete);
 
 	/* TS 44.060, B.8.1 */
-	cs = GprsCodingScheme::MCS4;
+	cs = MCS4;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 0;
@@ -348,7 +352,7 @@ static void test_rlc_unit_decoder()
 	 * includes the FBI/E header bits into the N2 octet count which
 	 * is not consistent with Section 10.3a.1 & 10.3a.2. */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 0;
@@ -412,7 +416,7 @@ static void test_rlc_unit_decoder()
 
 	/* Note that the spec confuses the byte numbering here, too (see above) */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 1;
 	rdbi.ti = 0;
@@ -428,7 +432,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(chunks[0].is_complete);
 
 	/* CS-1, TLLI, last block, single chunk until the end of the block */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 1;
 	rdbi.ti = 1;
@@ -449,7 +453,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(chunks[0].is_complete);
 
 	/* Like TS 44.060, B.2, first RLC block but with TLLI */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 1;
@@ -471,7 +475,7 @@ static void test_rlc_unit_decoder()
 	OSMO_ASSERT(!chunks[0].is_complete);
 
 	/* Like TS 44.060, B.8.1 but with TLLI */
-	cs = GprsCodingScheme::MCS4;
+	cs = MCS4;
 	rdbi.data_len = cs.maxDataBlockBytes();
 	rdbi.e = 0;
 	rdbi.ti = 1;
@@ -538,7 +542,7 @@ static void test_rlc_unit_encoder()
 	llc.init();
 
 	/* TS 44.060, B.1 */
-	cs = GprsCodingScheme::CS4;
+	cs = CS4;
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
 	num_chunks = 0;
 	write_offset = 0;
@@ -589,7 +593,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[2] == 0);
 
 	/* TS 44.060, B.2 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -649,7 +653,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[1] == 0);
 
 	/* TS 44.060, B.3 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -688,7 +692,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[2] == 0);
 
 	/* TS 44.060, B.4 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -713,7 +717,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[0] == 0);
 
 	/* TS 44.060, B.5 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -738,7 +742,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[0] == 0);
 
 	/* TS 44.060, B.7 */
-	cs = GprsCodingScheme::CS1;
+	cs = CS1;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -796,7 +800,7 @@ static void test_rlc_unit_encoder()
 	OSMO_ASSERT(data[1] == 0);
 
 	/* TS 44.060, B.8.1 */
-	cs = GprsCodingScheme::MCS4;
+	cs = MCS4;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -854,7 +858,7 @@ static void test_rlc_unit_encoder()
 	 * includes the FBI/E header bits into the N2 octet count which
 	 * is not consistent with Section 10.3a.1 & 10.3a.2. */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -986,7 +990,7 @@ static void test_rlc_unit_encoder()
 
 	/* Note that the spec confuses the byte numbering here, too (see above) */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -1012,7 +1016,7 @@ static void test_rlc_unit_encoder()
 
 	/* Final block with an LLC of size data_len-1 */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -1040,7 +1044,7 @@ static void test_rlc_unit_encoder()
 
 	/* Final block with an LLC of size data_len-2 */
 
-	cs = GprsCodingScheme::MCS2;
+	cs = MCS2;
 
 	/* Block 1 */
 	gprs_rlc_data_block_info_init(&rdbi, cs, false, 0);
@@ -1076,13 +1080,13 @@ static void test_rlc_unaligned_copy()
 	uint8_t saved_block[256];
 	uint8_t test_block[256];
 	uint8_t out_block[256];
-	GprsCodingScheme::Scheme scheme;
+	CodingScheme scheme;
 	int pattern;
 	volatile unsigned int block_idx, i;
 
-	for (scheme = GprsCodingScheme::CS1;
-		scheme < GprsCodingScheme::NUM_SCHEMES;
-		scheme = GprsCodingScheme::Scheme(scheme + 1))
+	for (scheme = CS1;
+		scheme < NUM_SCHEMES;
+		scheme = CodingScheme(scheme + 1))
 	{
 		GprsCodingScheme cs(scheme);
 
@@ -1094,7 +1098,7 @@ static void test_rlc_unaligned_copy()
 			test_block[cs.maxDataBlockBytes()-1] = pattern ^ 0xff;
 
 			for (block_idx = 0;
-				block_idx < cs.numDataBlocks();
+				block_idx < num_data_blocks(cs.headerTypeData());
 				block_idx++)
 			{
 				struct gprs_rlc_data_info rlc;
@@ -1114,7 +1118,7 @@ static void test_rlc_unaligned_copy()
 				fprintf(stderr,
 					"Encoded message block, %s, idx %d, "
 					"pattern %02x: %s\n",
-					rlc.cs.name(), block_idx, pattern,
+					mcs_name(rlc.cs), block_idx, pattern,
 					osmo_hexdump(bits, cs.sizeDL()));
 
 				Decoding::rlc_copy_to_aligned_buffer(
@@ -1143,13 +1147,13 @@ static void test_rlc_info_init()
 
 	printf("=== start %s ===\n", __func__);
 	gprs_rlc_data_info_init_dl(&rlc,
-			GprsCodingScheme(GprsCodingScheme::CS1), false, 0);
+			GprsCodingScheme(CS1), false, 0);
 	OSMO_ASSERT(rlc.num_data_blocks == 1);
 	OSMO_ASSERT(rlc.data_offs_bits[0] == 24);
 	OSMO_ASSERT(rlc.block_info[0].data_len == 20);
 
 	gprs_rlc_data_info_init_dl(&rlc,
-			GprsCodingScheme(GprsCodingScheme::MCS1), false, 0);
+			GprsCodingScheme(MCS1), false, 0);
 	OSMO_ASSERT(rlc.num_data_blocks == 1);
 	OSMO_ASSERT(rlc.data_offs_bits[0] == 33);
 	OSMO_ASSERT(rlc.block_info[0].data_len == 22);
@@ -1188,7 +1192,7 @@ static void uplink_header_type_2_parsing_test(BTS *the_bts,
 	int rc, offs;
 
 	/*without padding*/
-	cs = GprsCodingScheme::MCS5;
+	cs = MCS5;
 	egprs2 = (struct gprs_rlc_ul_header_egprs_2 *) data;
 	egprs2->r = 1;
 	egprs2->si = 1;
@@ -1213,7 +1217,7 @@ static void uplink_header_type_2_parsing_test(BTS *the_bts,
 	OSMO_ASSERT(rlc.block_info[0].bsn == 0);
 
 	/* with padding case */
-	cs = GprsCodingScheme::MCS6;
+	cs = MCS6;
 	egprs2 = (struct gprs_rlc_ul_header_egprs_2 *) data;
 	egprs2->r = 1;
 	egprs2->si = 1;
@@ -1293,7 +1297,7 @@ static void uplink_header_type_1_parsing_test(BTS *the_bts,
 	tfi = 1;
 
 	/* MCS 7 */
-	cs = GprsCodingScheme::MCS7;
+	cs = MCS7;
 	egprs1 = (struct gprs_rlc_ul_header_egprs_1 *) data;
 	egprs1->si = 1;
 	egprs1->r = 1;
@@ -1321,7 +1325,7 @@ static void uplink_header_type_1_parsing_test(BTS *the_bts,
 	OSMO_ASSERT(rlc.tfi == 1);
 
 	/* MCS 8 */
-	cs = GprsCodingScheme::MCS8;
+	cs = MCS8;
 	egprs1 = (struct gprs_rlc_ul_header_egprs_1 *) data;
 	egprs1->si = 1;
 	egprs1->r = 1;
@@ -1349,7 +1353,7 @@ static void uplink_header_type_1_parsing_test(BTS *the_bts,
 	OSMO_ASSERT(rlc.tfi == 1);
 
 	/* MCS 9 */
-	cs = GprsCodingScheme::MCS9;
+	cs = MCS9;
 	egprs1 = (struct gprs_rlc_ul_header_egprs_1 *) data;
 	egprs1->si = 1;
 	egprs1->r = 1;
@@ -1407,7 +1411,7 @@ int main(int argc, char **argv)
 	log_set_print_filename(osmo_stderr_target, 0);
 
 	vty_init(&pcu_vty_info);
-	pcu_vty_init(&gprs_log_info);
+	pcu_vty_init();
 
 	test_coding_scheme();
 	test_rlc_info_init();
